(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[    139758,       3328]
NotebookOptionsPosition[    136461,       3226]
NotebookOutlinePosition[    137823,       3262]
CellTagsIndexPosition[    137780,       3259]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Init", "Section",
 CellChangeTimes->{{3.5059460166723537`*^9, 3.505946018895583*^9}, {
  3.506042231454638*^9, 3.506042231843831*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"SetDirectory", "[", 
   RowBox[{"NotebookDirectory", "[", "]"}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Needs", "[", "\"\<Bulatov`dictionary`\>\"", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Needs", "[", "\"\<Bulatov`common`\>\"", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Needs", "[", "\"\<Bulatov`treeDecomposition`\>\"", "]"}], 
  ";"}]}], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Utils", "Section",
 CellChangeTimes->{{3.506042239240493*^9, 3.506042242811957*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"eps", "=", 
    RowBox[{"10", "^", 
     RowBox[{"-", "14"}]}]}], ";"}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"Hilbert", "'"}], "s", " ", "projective", " ", "metric"}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"projDist", "[", 
    RowBox[{"x_", ",", "y_"}], "]"}], ":=", 
   RowBox[{"(", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Assert", "[", 
      RowBox[{
       RowBox[{"VectorQ", "[", "x", "]"}], "&&", 
       RowBox[{"VectorQ", "[", "y", "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Log", "[", 
      RowBox[{
       RowBox[{"Max", "[", 
        RowBox[{"x", "/", "y"}], "]"}], "/", 
       RowBox[{"Min", "[", 
        RowBox[{"x", "/", "y"}], "]"}]}], "]"}]}], "\[IndentingNewLine]", 
    ")"}]}], "\[IndentingNewLine]", "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"compares", " ", "distance", " ", "between", " ", "messages"}], 
    ",", " ", 
    RowBox[{"discards", " ", "0", " ", "components"}]}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"msgDist", "[", 
     RowBox[{"msg1_", ",", "dom1_", ",", "msg2_", ",", "dom2_"}], "]"}], ":=", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Assert", "[", 
       RowBox[{
        RowBox[{"isUnnormProbTable", "[", "msg1", "]"}], "&&", 
        RowBox[{"isUnnormProbTable", "[", "msg2", "]"}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"ys1", "=", 
       RowBox[{
        RowBox[{"compact2fullTable", "[", 
         RowBox[{"msg1", ",", "dom1"}], "]"}], "//", "Last"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"ys2", "=", 
       RowBox[{
        RowBox[{"compact2fullTable", "[", 
         RowBox[{"msg2", ",", "dom2"}], "]"}], "//", "Last"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"posPoses", "=", 
       RowBox[{"Position", "[", 
        RowBox[{
         RowBox[{"ys1", " ", "ys2"}], ",", 
         RowBox[{"_", "?", "Positive"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"(*", 
       RowBox[{"projDist", "[", 
        RowBox[{
         RowBox[{"Extract", "[", 
          RowBox[{"ys1", ",", "posPoses"}], "]"}], ",", 
         RowBox[{"Extract", "[", 
          RowBox[{"ys2", ",", "posPoses"}], "]"}]}], "]"}], "*)"}], 
      "\[IndentingNewLine]", 
      RowBox[{"Total", "[", 
       RowBox[{"Abs", "[", 
        RowBox[{"ys1", "-", "ys2"}], "]"}], "]"}]}], "\[IndentingNewLine]", 
     ")"}]}], ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"approxZero", "[", "x_", "]"}], ":=", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"Chop", "[", "x", "]"}], "\[Equal]", "0"}], ")"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"entropy", "[", "ys_", "]"}], ":=", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Assert", "[", 
       RowBox[{"isProbDist", "[", "ys", "]"}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"relY", "=", 
       RowBox[{"Select", "[", 
        RowBox[{"ys", ",", "Positive"}], "]"}]}], ";", "\[IndentingNewLine]", 
      
      RowBox[{"-", 
       RowBox[{"Total", "[", 
        RowBox[{"relY", " ", 
         RowBox[{"Log", "[", "relY", "]"}]}], "]"}]}]}], 
     "\[IndentingNewLine]", ")"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"expectedEnergy", "[", 
     RowBox[{"probs_", ",", "pots_"}], "]"}], ":=", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Assert", "[", 
       RowBox[{"isProbDist", "[", "probs", "]"}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Assert", "[", 
       RowBox[{
        RowBox[{"\[LeftBracketingBar]", "probs", "\[RightBracketingBar]"}], 
        "\[Equal]", 
        RowBox[{"\[LeftBracketingBar]", "pots", "\[RightBracketingBar]"}]}], 
       "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"relProbs", "=", 
       RowBox[{"Select", "[", 
        RowBox[{"probs", ",", "Positive"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"relPots", "=", 
       RowBox[{"Extract", "[", 
        RowBox[{"pots", ",", 
         RowBox[{"Position", "[", 
          RowBox[{"probs", ",", 
           RowBox[{"_", "?", "Positive"}]}], "]"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Assert", "[", 
       RowBox[{"allTrue", "[", 
        RowBox[{
         RowBox[{"p", "\[Element]", "relPots"}], ",", 
         RowBox[{"p", ">", "0"}]}], "]"}], "]"}], ";", "\[IndentingNewLine]", 
      
      RowBox[{
       RowBox[{"Log", "[", "relPots", "]"}], ".", "relProbs"}]}], 
     "\[IndentingNewLine]", ")"}]}], ";"}], "\[IndentingNewLine]", "\n", 
  RowBox[{"(*", " ", 
   RowBox[{
   "gets", " ", "all", " ", "pairs", " ", "of", " ", "sets", " ", "that", " ",
     "intersect"}], " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"intersectingPairs", "[", "bags_List", "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"cands", ",", "result"}], "}"}], ",", "\n", "\t", 
     RowBox[{
      RowBox[{"cands", "=", 
       RowBox[{
        RowBox[{"Outer", "[", 
         RowBox[{"List", ",", "bags", ",", "bags", ",", "1"}], "]"}], "//", 
        "fl1"}]}], ";", "\n", "\t", 
      RowBox[{"Select", "[", 
       RowBox[{"cands", ",", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"#", "[", 
            RowBox[{"[", "1", "]"}], "]"}], "!=", 
           RowBox[{"#", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}], "&&", 
          RowBox[{"!", 
           RowBox[{"disjoint", "[", 
            RowBox[{
             RowBox[{"#", "[", 
              RowBox[{"[", "1", "]"}], "]"}], ",", 
             RowBox[{"#", "[", 
              RowBox[{"[", "2", "]"}], "]"}]}], "]"}]}], "&&", 
          RowBox[{"OrderedQ", "[", "#", "]"}]}], "&"}]}], "]"}]}]}], "\n", 
    "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{
  3.505946021001878*^9, {3.505951164581396*^9, 3.505951176614197*^9}, {
   3.5059518492873707`*^9, 3.505951932062883*^9}, {3.505952387600759*^9, 
   3.5059524010932713`*^9}, {3.505961226739348*^9, 3.50596123402792*^9}, {
   3.505963325665374*^9, 3.505963390520082*^9}, {3.505963537344901*^9, 
   3.50596356260215*^9}, {3.505963603906143*^9, 3.505963624925982*^9}, {
   3.5059638803388443`*^9, 3.505963929131267*^9}, {3.505964008585348*^9, 
   3.505964038510911*^9}, {3.5059640725258636`*^9, 3.5059640750512667`*^9}, {
   3.505964106371175*^9, 3.50596411956822*^9}, {3.505967612925034*^9, 
   3.505967629783332*^9}, {3.5059677684991217`*^9, 3.505967812473733*^9}, {
   3.5059679098646393`*^9, 3.505967950157735*^9}, {3.505968454816925*^9, 
   3.505968456368931*^9}, {3.505968702419939*^9, 3.50596872671904*^9}, {
   3.505968771179463*^9, 3.5059688270380793`*^9}, {3.505968893793654*^9, 
   3.505968976785336*^9}, {3.506004110415379*^9, 3.506004226276742*^9}, {
   3.50600435664604*^9, 3.506004358983667*^9}, {3.506004556387577*^9, 
   3.5060045589912567`*^9}, {3.506004696402446*^9, 3.506004698453071*^9}, {
   3.5060053957754993`*^9, 3.506005398099141*^9}, {3.506005458463242*^9, 
   3.506005485907774*^9}, {3.506009639988029*^9, 3.506009655460013*^9}, {
   3.5060096917689533`*^9, 3.506009728054027*^9}, {3.5060098098423653`*^9, 
   3.506009810015221*^9}, {3.506010196113844*^9, 3.506010207537498*^9}, {
   3.506044082864246*^9, 3.50604408689774*^9}, {3.506044171369025*^9, 
   3.506044178449877*^9}, {3.506052363154455*^9, 3.506052363585485*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Setup", "Section",
 CellChangeTimes->{{3.505847090950367*^9, 3.505847094342091*^9}, {
  3.505851614235525*^9, 3.505851614609065*^9}}],

Cell["\<\
g, verts, edges, vcoords, nativeFactors, cl
messages, cX,cY,cD\
\>", "Text",
 CellChangeTimes->{{3.5058470975102158`*^9, 3.505847106500839*^9}, {
  3.505849054644403*^9, 3.505849055139452*^9}, {3.505849167557938*^9, 
  3.50584919768426*^9}, {3.505851049150511*^9, 3.505851049518058*^9}, {
  3.50596832096845*^9, 3.50596835155378*^9}}],

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{"factor", " ", "and", " ", "clique", " ", "children"}], " ", 
   "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"ch", "[", 
      RowBox[{"{", "i_List", "}"}], "]"}], ":=", 
     RowBox[{"Cases", "[", 
      RowBox[{"messages", ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "_"}], "}"}]}], "]"}]}], ";"}], "\n", 
   RowBox[{
    RowBox[{
     RowBox[{"ch", "[", 
      RowBox[{"{", 
       RowBox[{"i_List", ",", "j_List"}], "}"}], "]"}], ":=", 
     RowBox[{"Cases", "[", 
      RowBox[{"messages", ",", 
       RowBox[{"{", 
        RowBox[{"j", ",", 
         RowBox[{"Except", "[", "i", "]"}]}], "}"}]}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", "\n", 
   RowBox[{"(*", " ", 
    RowBox[{
     RowBox[{"initializes", " ", "graph"}], ",", " ", 
     RowBox[{"and", " ", "graphical", " ", "model", " ", "factors"}]}], " ", 
    "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"initHardcore", "[", "graph_", "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Assert", "[", 
        RowBox[{
         RowBox[{"Head", "[", "graph", "]"}], "===", "Graph"}], "]"}], ";", 
       "\[IndentingNewLine]", "\[IndentingNewLine]", 
       RowBox[{"g", "=", "graph"}], ";", "\[IndentingNewLine]", 
       RowBox[{"verts", "=", 
        RowBox[{"VertexList", "[", "g", "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"edges", "=", 
        RowBox[{"deepSort", "[", 
         RowBox[{"List", "@@@", 
          RowBox[{"EdgeList", "[", "g", "]"}]}], "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"vcoords", "=", 
        RowBox[{"VertexCoordinates", "/.", 
         RowBox[{"AbsoluteOptions", "[", 
          RowBox[{"g", ",", "VertexCoordinates"}], "]"}]}]}], ";", 
       "\[IndentingNewLine]", "\[IndentingNewLine]", 
       RowBox[{"nativeFactors", "=", 
        RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"Clear", "[", 
        RowBox[{"cX", ",", "cY", ",", "cD"}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"setupVertexFactor", "/@", "verts"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"setupEdgeFactor", "/@", "edges"}], ";"}], 
      "\[IndentingNewLine]", ")"}]}], ";"}], "\[IndentingNewLine]", 
   "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
    "initializes", " ", "message", " ", "passing", " ", "scheme", " ", "with",
      " ", "given", " ", "set", " ", "of", " ", "regions"}], " ", "*)"}], 
   "\n", 
   RowBox[{
    RowBox[{
     RowBox[{"initMessages", "[", 
      RowBox[{"{", 
       RowBox[{"dnodes_", ",", "dedges_"}], "}"}], "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{"jnodes", ",", "jedges"}], "}"}], "=", 
        RowBox[{"{", 
         RowBox[{"dnodes", ",", "dedges"}], "}"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"assignFactors", "[", "nativeFactors", "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"messages", "=", 
        RowBox[{"jedges", "\[Tilde]", 
         RowBox[{"(", 
          RowBox[{"Reverse", "/@", "jedges"}], ")"}]}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"setupMessageFactor", "/@", "messages"}], ";"}], 
      "\[IndentingNewLine]", ")"}]}], ";"}], "\[IndentingNewLine]", 
   "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
    "greedily", " ", "assign", " ", "factors", " ", "to", " ", "regions"}], 
    " ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"assignFactors", "[", "factors_", "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"assignToBag", "[", "jnode_", "]"}], ":=", 
        RowBox[{"(", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"coveredFactors", "=", 
           RowBox[{"Select", "[", 
            RowBox[{"unassigned", ",", 
             RowBox[{
              RowBox[{"#", "\[Subset]", "jnode"}], "&"}]}], "]"}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"cl", "[", "jnode", "]"}], "=", "coveredFactors"}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"unassigned", "=", 
           RowBox[{"unassigned", "\[Backslash]", "coveredFactors"}]}], ";"}], 
         "\[IndentingNewLine]", ")"}]}], ";", "\[IndentingNewLine]", 
       "\[IndentingNewLine]", 
       RowBox[{"Clear", "[", "cl", "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{"unassigned", "=", "factors"}], ";", "\[IndentingNewLine]", 
       RowBox[{"assignToBag", "/@", "jnodes"}], ";", "\[IndentingNewLine]", 
       RowBox[{"Assert", "[", 
        RowBox[{"unassigned", "==", 
         RowBox[{"{", "}"}]}], "]"}], ";"}], "\[IndentingNewLine]", ")"}]}], 
    ";"}], "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"setupVertexFactor", "[", "v_Integer", "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"AppendTo", "[", 
        RowBox[{"nativeFactors", ",", 
         RowBox[{"{", "v", "}"}]}], "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"cD", "[", 
         RowBox[{"{", "v", "}"}], "]"}], "=", 
        RowBox[{"{", "v", "}"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"cX", "[", 
         RowBox[{"{", "v", "}"}], "]"}], "=", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", "}"}], ",", 
          RowBox[{"{", "v", "}"}]}], "}"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"cY", "[", 
         RowBox[{"{", "v", "}"}], "]"}], "=", 
        RowBox[{"{", 
         RowBox[{"1", ",", "\[Lambda]"}], "}"}]}], ";"}], 
      "\[IndentingNewLine]", ")"}]}], ";"}], "\n", "\n", 
   RowBox[{
    RowBox[{
     RowBox[{"setupEdgeFactor", "[", "ef_List", "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"AppendTo", "[", 
        RowBox[{"nativeFactors", ",", "ef"}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"cD", "[", "ef", "]"}], "=", "ef"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"cX", "[", "ef", "]"}], "=", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"First", "[", "ef", "]"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"Last", "[", "ef", "]"}], "}"}]}], "}"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"cY", "[", "ef", "]"}], "=", 
        RowBox[{"{", 
         RowBox[{"1", ",", "1", ",", "1"}], "}"}]}], ";"}], "\n", ")"}]}], 
    ";"}], "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
    "initialize", " ", "messages", " ", "to", " ", "constant", " ", 
     "factors"}], " ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"setupMessageFactor", "[", 
      RowBox[{"{", 
       RowBox[{"i_", ",", "j_"}], "}"}], "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"dom", "=", 
        RowBox[{"i", "\[Intersection]", "j"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"cD", "[", 
         RowBox[{"{", 
          RowBox[{"i", ",", "j"}], "}"}], "]"}], "=", "dom"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"cX", "[", 
         RowBox[{"{", 
          RowBox[{"i", ",", "j"}], "}"}], "]"}], "=", 
        RowBox[{"Subsets", "[", "dom", "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"(*", 
        RowBox[{
         RowBox[{
          RowBox[{"cY", "[", 
           RowBox[{"{", 
            RowBox[{"i", ",", "j"}], "}"}], "]"}], "=", 
          RowBox[{"ConstantArray", "[", 
           RowBox[{"1", ",", 
            RowBox[{"Length", "[", 
             RowBox[{"cX", "[", 
              RowBox[{"{", 
               RowBox[{"i", ",", "j"}], "}"}], "]"}], "]"}]}], "]"}]}], ";"}],
         "*)"}], "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"cY", "[", 
         RowBox[{"{", 
          RowBox[{"i", ",", "j"}], "}"}], "]"}], "=", 
        RowBox[{"RandomReal", "[", 
         RowBox[{"1", ",", 
          RowBox[{"Length", "[", 
           RowBox[{"cX", "[", 
            RowBox[{"{", 
             RowBox[{"i", ",", "j"}], "}"}], "]"}], "]"}]}], "]"}]}]}], "\n", 
      ")"}]}], ";"}], "\[IndentingNewLine]"}]}]], "Input",
 CellChangeTimes->{
  3.50584713519993*^9, {3.505848938480151*^9, 3.50584906461322*^9}, {
   3.505849136524498*^9, 3.5058491585500317`*^9}, {3.505849424963636*^9, 
   3.505849482466947*^9}, {3.505849541688121*^9, 3.5058495482938747`*^9}, {
   3.505850465867388*^9, 3.505850556472191*^9}, {3.5058505899005537`*^9, 
   3.505850601216628*^9}, 3.505850704433179*^9, {3.505850767935946*^9, 
   3.505850777422864*^9}, {3.505850910118185*^9, 3.5058509109961653`*^9}, {
   3.50585096097962*^9, 3.5058509619011707`*^9}, {3.505851004426957*^9, 
   3.5058510632776537`*^9}, {3.5058511370499268`*^9, 
   3.5058511767505093`*^9}, {3.505851515561326*^9, 3.50585155176222*^9}, {
   3.505851775102766*^9, 3.505851775642047*^9}, {3.5058537318421*^9, 
   3.5058537441397676`*^9}, {3.505854722217517*^9, 3.505854723655239*^9}, {
   3.505855447231573*^9, 3.505855459872963*^9}, {3.5058567412478437`*^9, 
   3.505856744456108*^9}, {3.505856872479044*^9, 3.50585687329599*^9}, {
   3.505860284137576*^9, 3.505860293259658*^9}, {3.505861085870616*^9, 
   3.505861118492197*^9}, {3.505876026867579*^9, 3.505876084455124*^9}, {
   3.505876117574988*^9, 3.505876130464717*^9}, {3.505877319945837*^9, 
   3.5058773273992662`*^9}, {3.505955974861356*^9, 3.505955990394339*^9}, {
   3.5059561110618763`*^9, 3.505956112492885*^9}, {3.505967401489799*^9, 
   3.505967470443736*^9}, {3.5059675008679113`*^9, 3.5059675369026403`*^9}, 
   3.505967764302567*^9, {3.5060534839851713`*^9, 3.5060535265054197`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Structures", "Section",
 CellChangeTimes->{{3.505968689095542*^9, 3.505968693488298*^9}, {
  3.5059688352702827`*^9, 3.505968837319168*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{
   "Messages", " ", "are", " ", "sparse", " ", "probability", " ", "tables"}],
    " ", "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"isProbTable", "[", "table_", "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"\[LeftBracketingBar]", "table", "\[RightBracketingBar]"}], 
        "\[Equal]", "2"}], "&&", 
       RowBox[{
        RowBox[{"\[LeftBracketingBar]", 
         RowBox[{
         "table", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
         "\[RightBracketingBar]"}], "\[Equal]", 
        RowBox[{"\[LeftBracketingBar]", 
         RowBox[{
         "table", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
         "\[RightBracketingBar]"}]}], "&&", 
       RowBox[{"VectorQ", "[", 
        RowBox[{
        "table", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
        "]"}], "&&", 
       RowBox[{"allTrue", "[", 
        RowBox[{
         RowBox[{"k", "\[Element]", 
          RowBox[{
          "table", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]}], 
         ",", 
         RowBox[{"k", "\[GreaterEqual]", "0"}]}], "]"}], "&&", 
       RowBox[{"approxZero", "[", 
        RowBox[{
         RowBox[{"Total", "[", 
          RowBox[{
          "table", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
          "]"}], "-", "1"}], "]"}]}], "\[IndentingNewLine]", ")"}]}], ";"}], 
   "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"isUnnormProbTable", "[", "table_", "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"\[LeftBracketingBar]", "table", "\[RightBracketingBar]"}], 
        "\[Equal]", "2"}], "&&", 
       RowBox[{
        RowBox[{"\[LeftBracketingBar]", 
         RowBox[{
         "table", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
         "\[RightBracketingBar]"}], "\[Equal]", 
        RowBox[{"\[LeftBracketingBar]", 
         RowBox[{
         "table", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
         "\[RightBracketingBar]"}]}], "&&", 
       RowBox[{"VectorQ", "[", 
        RowBox[{
        "table", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
        "]"}], "&&", 
       RowBox[{"allTrue", "[", 
        RowBox[{
         RowBox[{"k", "\[Element]", 
          RowBox[{
          "table", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]}], 
         ",", 
         RowBox[{"k", "\[GreaterEqual]", "0"}]}], "]"}]}], 
      "\[IndentingNewLine]", ")"}]}], ";"}], "\[IndentingNewLine]", 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"isProbDist", "[", "dist_", "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"allTrue", "[", 
        RowBox[{
         RowBox[{"d", "\[Element]", "dist"}], ",", 
         RowBox[{"d", "\[GreaterEqual]", "0"}]}], "]"}], "&&", 
       RowBox[{"approxZero", "[", 
        RowBox[{
         RowBox[{"Total", "@", "dist"}], "-", "1"}], "]"}]}], 
      "\[IndentingNewLine]", ")"}]}], ";"}], "\[IndentingNewLine]", 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"getXS", "[", "table_", "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Assert", "[", 
        RowBox[{"isUnnormProbTable", "[", "table", "]"}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"First", "[", "table", "]"}]}], "\[IndentingNewLine]", ")"}]}],
     ";"}], "\n", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"getYS", "[", "table_", "]"}], ":=", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Assert", "[", 
       RowBox[{"isUnnormProbTable", "[", "table", "]"}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Last", "[", "table", "]"}]}], "\[IndentingNewLine]", ")"}]}], 
   "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"normalizeTable", "[", "table_", "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Assert", "[", 
        RowBox[{"isUnnormProbTable", "@", "table"}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"xs", ",", "ys"}], "}"}], "=", "table"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"newYS", "=", 
        RowBox[{"Normalize", "[", 
         RowBox[{"ys", ",", "Total"}], "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"Assert", "[", 
        RowBox[{"isProbTable", "[", 
         RowBox[{"{", 
          RowBox[{"xs", ",", "newYS"}], "}"}], "]"}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"xs", ",", "newYS"}], "}"}]}], "\[IndentingNewLine]", ")"}]}],
     ";"}], "\n", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"multiplyTables", "[", 
      RowBox[{"table1_", ",", "table2_"}], "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"sharedX", "=", 
        RowBox[{
         RowBox[{"getXS", "[", "table1", "]"}], "\[Intersection]", 
         RowBox[{"getXS", "[", "table2", "]"}]}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"ys1", "=", 
        RowBox[{"Extract", "[", 
         RowBox[{
          RowBox[{"getYS", "@", "table1"}], ",", 
          RowBox[{"Position", "[", 
           RowBox[{
            RowBox[{"getXS", "@", "table1"}], ",", 
            RowBox[{"Alternatives", "@@", "sharedX"}]}], "]"}]}], "]"}]}], 
       ";", "\[IndentingNewLine]", 
       RowBox[{"ys2", "=", 
        RowBox[{"Extract", "[", 
         RowBox[{
          RowBox[{"getYS", "@", "table2"}], ",", 
          RowBox[{"Position", "[", 
           RowBox[{
            RowBox[{"getXS", "@", "table2"}], ",", 
            RowBox[{"Alternatives", "@@", "sharedX"}]}], "]"}]}], "]"}]}], 
       ";", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"sharedX", ",", 
         RowBox[{"ys1", " ", "ys2"}]}], "}"}]}], "\[IndentingNewLine]", 
      ")"}]}], ";"}], "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
    "converts", " ", "clause", " ", "representation", " ", "to", " ", "full", 
     " ", "form", " ", "with", " ", "explicit", " ", "negations"}], " ", 
    "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"compact2full", "[", 
      RowBox[{"trueVars_List", ",", "allVars_List"}], "]"}], ":=", 
     RowBox[{"Module", "[", 
      RowBox[{
       RowBox[{"{", "negate", "}"}], ",", "\n", "\t", 
       RowBox[{
        RowBox[{
         RowBox[{"negate", "[", "lst_", "]"}], ":=", 
         RowBox[{"Not", "/@", "lst"}]}], ";", "\n", "\t", 
        RowBox[{"And", "@@", 
         RowBox[{"(", 
          RowBox[{"trueVars", "\[Tilde]", 
           RowBox[{"negate", "[", 
            RowBox[{"allVars", "\[Backslash]", "trueVars"}], "]"}]}], 
          ")"}]}]}]}], "\n", "]"}]}], ";"}], "\[IndentingNewLine]", 
   "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
     RowBox[{"gives", " ", "non"}], "-", 
     RowBox[{
     "sparse", " ", "representation", " ", "of", " ", "probability", " ", 
      "table"}]}], " ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"compact2fullTable", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"xs_", ",", "ys_"}], "}"}], ",", "dom_"}], "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"xsAll", "=", 
        RowBox[{"Subsets", "[", "dom", "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"ysAll", "=", 
        RowBox[{"ys", "\[Tilde]", 
         RowBox[{"ConstantArray", "[", 
          RowBox[{"0", ",", 
           RowBox[{"\[LeftBracketingBar]", 
            RowBox[{"xsAll", "\[Backslash]", "xs"}], 
            "\[RightBracketingBar]"}]}], "]"}]}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"xs", "\[Tilde]", 
          RowBox[{"(", 
           RowBox[{"xsAll", "\[Backslash]", "xs"}], ")"}]}], ",", "ysAll"}], 
        "}"}]}], "\[IndentingNewLine]", ")"}]}], ";"}], "\[IndentingNewLine]",
    "\n"}]}]], "Input",
 CellChangeTimes->{{3.505968839478455*^9, 3.505968884029011*^9}, {
  3.5059701070080976`*^9, 3.505970154663919*^9}, {3.5059702228496304`*^9, 
  3.505970226566452*^9}, {3.506010336204357*^9, 3.50601033660561*^9}, {
  3.506011095830452*^9, 3.5060111634526367`*^9}, {3.506011196835463*^9, 
  3.506011294480282*^9}, {3.506011909573279*^9, 3.506011967078443*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Visualization utils", "Section",
 CellChangeTimes->{{3.505969574205971*^9, 3.5059695793565598`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", 
   RowBox[{
    RowBox[{"**", " ", "helper"}], " ", "utils", " ", "for", " ", 
    "visualization"}], " ", "***)"}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
   "gets", " ", "list", " ", "of", " ", "occupation", " ", "probabilities", 
    " ", "of", " ", "nodes", " ", "in", " ", "the", " ", "clique"}], " ", 
   "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"cliqueMarginals", "[", "clique_", "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Assert", "[", 
        RowBox[{"clique", "\[Element]", "jnodes"}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"vals", "=", 
        RowBox[{"evalClique", "@", "clique"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"xs", ",", "ys"}], "}"}], "=", 
        RowBox[{"normalizeTable", "@", 
         RowBox[{"compact2fullTable", "[", 
          RowBox[{"vals", ",", "clique"}], "]"}]}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"(*", " ", 
        RowBox[{
        "converts", " ", "marginal", " ", "distribution", " ", "into", " ", 
         "occupation", " ", "probability"}], " ", "*)"}], 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"dist2occ", "[", 
         RowBox[{"{", 
          RowBox[{"xss_", ",", "yss_"}], "}"}], "]"}], ":=", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Assert", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"First", "@", "xss"}], "\[Equal]", 
             RowBox[{"{", "}"}]}], "&&", 
            RowBox[{
             RowBox[{"\[LeftBracketingBar]", "xss", "\[RightBracketingBar]"}],
              "\[Equal]", "2"}]}], "]"}], ";", 
          RowBox[{
          "yss", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]}], 
         ")"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"dist2occ", "@", 
          RowBox[{"evalSum", "[", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"xs", ",", "ys"}], "}"}], ",", 
            RowBox[{"{", "#", "}"}]}], "]"}]}], "&"}], "/@", "clique"}]}], 
      "\[IndentingNewLine]", ")"}]}], ";"}], "\[IndentingNewLine]", 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"nodeMarginals", "[", "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Clear", "[", "margMap", "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"saveMarginals", "[", "clique_", "]"}], ":=", 
        RowBox[{"(", "\[IndentingNewLine]", 
         RowBox[{"MapThread", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{"margMap", "[", "#1", "]"}], "=", "#2"}], ")"}], "&"}], 
           ",", 
           RowBox[{"{", 
            RowBox[{"clique", ",", 
             RowBox[{"cliqueMarginals", "[", "clique", "]"}]}], "}"}]}], 
          "]"}], "\[IndentingNewLine]", ")"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"saveMarginals", "/@", "jnodes"}], ";", "\[IndentingNewLine]", 
       
       RowBox[{
        RowBox[{"Sort", "[", 
         RowBox[{"items", "[", "margMap", "]"}], "]"}], "[", 
        RowBox[{"[", 
         RowBox[{"All", ",", "2"}], "]"}], "]"}]}], "\[IndentingNewLine]", 
      ")"}]}], ";"}], "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"getNodeMarginalsCurrent", "[", "edge_", "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Assert", "[", 
        RowBox[{"edge", "\[Element]", "jedges"}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"t1", ",", "t2"}], "}"}], "=", 
        RowBox[{"evalCurrentMsg", "/@", 
         RowBox[{"{", 
          RowBox[{"edge", ",", 
           RowBox[{"Reverse", "@", "edge"}]}], "}"}]}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"sep", "=", 
        RowBox[{"Intersection", "@@", "edge"}]}], ";", "\[IndentingNewLine]", 
       
       RowBox[{"msgProd", "=", 
        RowBox[{"normalizeTable", "@", 
         RowBox[{"multiplyTables", "[", 
          RowBox[{"t1", ",", "t2"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"(*", " ", 
        RowBox[{
        "convert", " ", "distribution", " ", "to", " ", "occupation", " ", 
         "probability"}], " ", "*)"}], "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"dist2occ", "[", 
         RowBox[{"{", 
          RowBox[{"xss_", ",", "yss_"}], "}"}], "]"}], ":=", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Assert", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"First", "@", "xss"}], "\[Equal]", 
             RowBox[{"{", "}"}]}], "&&", 
            RowBox[{
             RowBox[{"\[LeftBracketingBar]", "xss", "\[RightBracketingBar]"}],
              "\[Equal]", "2"}], "&&", 
            RowBox[{
             RowBox[{"Total", "[", "yss", "]"}], "\[Equal]", "1"}]}], "]"}], 
          ";", 
          RowBox[{
          "yss", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]}], 
         ")"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"dist2occ", "@", 
          RowBox[{"evalSum", "[", 
           RowBox[{"msgProd", ",", 
            RowBox[{"{", "#", "}"}]}], "]"}]}], "&"}], "/@", "sep"}]}], 
      "\[IndentingNewLine]", ")"}]}], ";"}], "\[IndentingNewLine]", 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"getNodeMarginals", "[", "edge_", "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Assert", "[", 
        RowBox[{"edge", "\[Element]", "jedges"}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"t1", ",", "t2"}], "}"}], "=", 
        RowBox[{"evalMsg", "/@", 
         RowBox[{"{", 
          RowBox[{"edge", ",", 
           RowBox[{"Reverse", "@", "edge"}]}], "}"}]}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"sep", "=", 
        RowBox[{"Intersection", "@@", "edge"}]}], ";", "\[IndentingNewLine]", 
       
       RowBox[{"msgProd", "=", 
        RowBox[{"normalizeTable", "@", 
         RowBox[{"multiplyTables", "[", 
          RowBox[{"t1", ",", "t2"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"(*", " ", 
        RowBox[{
        "convert", " ", "distribution", " ", "to", " ", "occupation", " ", 
         "probability"}], " ", "*)"}], "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"dist2occ", "[", 
         RowBox[{"{", 
          RowBox[{"xss_", ",", "yss_"}], "}"}], "]"}], ":=", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Assert", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"First", "@", "xss"}], "\[Equal]", 
             RowBox[{"{", "}"}]}], "&&", 
            RowBox[{
             RowBox[{"\[LeftBracketingBar]", "xss", "\[RightBracketingBar]"}],
              "\[Equal]", "2"}], "&&", 
            RowBox[{
             RowBox[{"Total", "[", "yss", "]"}], "\[Equal]", "1"}]}], "]"}], 
          ";", 
          RowBox[{
          "yss", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]}], 
         ")"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"dist2occ", "@", 
          RowBox[{"evalSum", "[", 
           RowBox[{"msgProd", ",", 
            RowBox[{"{", "#", "}"}]}], "]"}]}], "&"}], "/@", "sep"}]}], 
      "\[IndentingNewLine]", ")"}]}], ";"}], "\[IndentingNewLine]", 
   "\[IndentingNewLine]", 
   RowBox[{"(*", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{"xs", ",", "ys"}], "}"}], "=", 
      RowBox[{"normalizeTable", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"cX", "[", "edge", "]"}], ",", "prodY"}], "}"}], "]"}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"dist2occ", "[", 
       RowBox[{"{", 
        RowBox[{"xss_", ",", "yss_"}], "}"}], "]"}], ":=", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"Assert", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"First", "@", "xss"}], "\[Equal]", 
           RowBox[{"{", "}"}]}], "&&", 
          RowBox[{
           RowBox[{"\[LeftBracketingBar]", "xss", "\[RightBracketingBar]"}], 
           "\[Equal]", "2"}], "&&", 
          RowBox[{
           RowBox[{"Total", "[", "ys", "]"}], "\[Equal]", "1"}]}], "]"}], ";", 
        RowBox[{
        "yss", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]}], 
       ")"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{
       RowBox[{"dist2occ", "@", 
        RowBox[{"evalSum", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"xs", ",", "ys"}], "}"}], ",", 
          RowBox[{"{", "#", "}"}]}], "]"}]}], "&"}], "/@", "sep"}]}], "*)"}], 
   "\[IndentingNewLine]", "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"twoColorGraph", "[", 
      RowBox[{
      "vals1_", ",", "vals2_", ",", "verts_", ",", "edges_", ",", 
       "vcoords_"}], "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"vf", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"xc_", ",", "yc_"}], "}"}], ",", "name_", ",", 
          RowBox[{"{", 
           RowBox[{"w_", ",", "h_"}], "}"}]}], "]"}], ":=", 
        RowBox[{"(", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"d1", "=", 
           RowBox[{"Disk", "[", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"xc", ",", "yc"}], "}"}], ",", 
             RowBox[{"Max", "[", 
              RowBox[{"w", ",", "h"}], "]"}], ",", 
             RowBox[{"{", 
              RowBox[{"0", ",", "Pi"}], "}"}]}], "]"}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"d2", "=", 
           RowBox[{"Disk", "[", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"xc", ",", "yc"}], "}"}], ",", 
             RowBox[{"Max", "[", 
              RowBox[{"w", ",", "h"}], "]"}], ",", 
             RowBox[{"{", 
              RowBox[{"Pi", ",", 
               RowBox[{"2", "Pi"}]}], "}"}]}], "]"}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{
             RowBox[{"ColorData", "[", "\"\<Crayola\>\"", "]"}], "[", 
             "\"\<NavyBlue\>\"", "]"}], ",", 
            RowBox[{"Opacity", "[", 
             RowBox[{
             "vals1", "\[LeftDoubleBracket]", "name", 
              "\[RightDoubleBracket]"}], "]"}], ",", "d1", ",", 
            RowBox[{"Opacity", "[", 
             RowBox[{
             "vals2", "\[LeftDoubleBracket]", "name", 
              "\[RightDoubleBracket]"}], "]"}], ",", "d2"}], "}"}]}], 
         "\[IndentingNewLine]", ")"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"Graph", "[", 
        RowBox[{"verts", ",", 
         RowBox[{"UndirectedEdge", "@@@", "edges"}], ",", 
         RowBox[{"VertexShapeFunction", "\[Rule]", "vf"}], ",", 
         RowBox[{"VertexSize", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{"\"\<Scaled\>\"", ",", ".1"}], "}"}]}], ",", 
         RowBox[{"VertexCoordinates", "\[Rule]", "vcoords"}]}], "]"}]}], 
      "\[IndentingNewLine]", ")"}]}], ";"}], "\[IndentingNewLine]", 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"oneColorGraph", "[", 
      RowBox[{"vals1_", ",", "verts_", ",", "edges_", ",", "vcoords_"}], 
      "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"vf", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"xc_", ",", "yc_"}], "}"}], ",", "name_", ",", 
          RowBox[{"{", 
           RowBox[{"w_", ",", "h_"}], "}"}]}], "]"}], ":=", 
        RowBox[{"(", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"d1", "=", 
           RowBox[{"Disk", "[", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"xc", ",", "yc"}], "}"}], ",", 
             RowBox[{"Max", "[", 
              RowBox[{"w", ",", "h"}], "]"}], ",", 
             RowBox[{"{", 
              RowBox[{"0", ",", 
               RowBox[{"2", "Pi"}]}], "}"}]}], "]"}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{
             RowBox[{"ColorData", "[", "\"\<Crayola\>\"", "]"}], "[", 
             "\"\<NavyBlue\>\"", "]"}], ",", 
            RowBox[{"Opacity", "[", 
             RowBox[{
             "vals1", "\[LeftDoubleBracket]", "name", 
              "\[RightDoubleBracket]"}], "]"}], ",", "d1"}], "}"}]}], 
         "\[IndentingNewLine]", ")"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"Graph", "[", 
        RowBox[{"verts", ",", 
         RowBox[{"UndirectedEdge", "@@@", "edges"}], ",", 
         RowBox[{"VertexShapeFunction", "\[Rule]", "vf"}], ",", 
         RowBox[{"VertexSize", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{"\"\<Scaled\>\"", ",", ".1"}], "}"}]}], ",", 
         RowBox[{"VertexCoordinates", "\[Rule]", "vcoords"}]}], "]"}]}], 
      "\[IndentingNewLine]", ")"}]}], ";"}], "\[IndentingNewLine]", 
   "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{"gets", " ", "average", " ", "marginals"}], " ", "*)"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"aveMarginals", "[", "margFunc_", "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"nodes", "=", 
        RowBox[{"Intersection", "@@@", "jedges"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"marg", "=", 
        RowBox[{"margFunc", "/@", "jedges"}]}], ";", "\[IndentingNewLine]", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"averageGroup", "[", "gr_", "]"}], ":=", 
        RowBox[{"(", "\[IndentingNewLine]", 
         RowBox[{"Mean", "[", 
          RowBox[{"gr", "[", 
           RowBox[{"[", 
            RowBox[{"All", ",", "2"}], "]"}], "]"}], "]"}], 
         "\[IndentingNewLine]", ")"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"averageGroup", "/@", 
        RowBox[{"GatherBy", "[", 
         RowBox[{
          RowBox[{"fl1", "@", 
           RowBox[{"Transpose", "[", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"nodes", ",", "marg"}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"3", ",", "2", ",", "1"}], "}"}]}], "]"}]}], ",", 
          "First"}], "]"}]}]}], "\[IndentingNewLine]", "\[IndentingNewLine]", 
      ")"}]}], ";"}], "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"visualizeCavities", "[", "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"oldVals", "=", 
        RowBox[{"aveMarginals", "[", "getNodeMarginalsCurrent", "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"newVals", "=", 
        RowBox[{"aveMarginals", "[", "getNodeMarginals", "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"Show", "@", 
        RowBox[{"twoColorGraph", "[", 
         RowBox[{
         "oldVals", ",", "newVals", ",", "verts", ",", "edges", ",", 
          "vcoords"}], "]"}]}]}], "\[IndentingNewLine]", ")"}]}], ";"}], 
   "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"visualizeCavities2", "[", "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"vals", "=", 
        RowBox[{"nodeMarginals", "[", "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"Show", "@", 
        RowBox[{"oneColorGraph", "[", 
         RowBox[{"vals", ",", "verts", ",", "edges", ",", "vcoords"}], 
         "]"}]}]}], "\[IndentingNewLine]", ")"}]}], ";"}]}]}]], "Input",
 CellChangeTimes->{{3.50604852677833*^9, 3.506048527068293*^9}, {
   3.5060485865132103`*^9, 3.5060486460259123`*^9}, {3.506048703604124*^9, 
   3.506048712291614*^9}, {3.506048782379575*^9, 3.506048793864998*^9}, {
   3.506048921582398*^9, 3.50604893787897*^9}, {3.506049138640099*^9, 
   3.506049186021927*^9}, {3.5060494261498938`*^9, 3.506049445387047*^9}, {
   3.5060502095825243`*^9, 3.506050257210494*^9}, 3.5060503082660513`*^9, {
   3.506050438388348*^9, 3.506050438748516*^9}, {3.5060512551145144`*^9, 
   3.506051266501548*^9}, {3.506051489306644*^9, 3.5060514922682867`*^9}, {
   3.506051625523037*^9, 3.50605162584962*^9}, {3.506051679917388*^9, 
   3.5060516803021517`*^9}, {3.5060527293634033`*^9, 3.506052778077642*^9}, {
   3.506053863665402*^9, 3.506053878752553*^9}, {3.506054118097981*^9, 
   3.506054172428011*^9}, {3.506054544031254*^9, 3.506054560906207*^9}, 
   3.506054596297556*^9, {3.5060546277464523`*^9, 3.506054683826997*^9}, {
   3.506054720889216*^9, 3.506054757012096*^9}, {3.5060562041911383`*^9, 
   3.5060562109471684`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Iterate", "Section",
 CellChangeTimes->{{3.505851618572435*^9, 3.505851620826538*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"parallelUpdate", "[", "1", "]"}], ":=", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"temp", "=", 
       RowBox[{"evalMsg", "/@", "messages"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"MapThread", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"setMsg", "[", 
          RowBox[{"#1", ",", "#2"}], "]"}], "&"}], ",", 
        RowBox[{"{", 
         RowBox[{"messages", ",", "temp"}], "}"}]}], "]"}], ";"}], 
     "\[IndentingNewLine]", ")"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"parallelUpdate", "[", "k_", "]"}], ":=", 
   RowBox[{"(", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Assert", "[", 
      RowBox[{"k", "\[Element]", "Integers"}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"parallelUpdate", "[", "1", "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"j", ",", "1", ",", "k"}], "}"}]}], "]"}], ";"}], 
    "\[IndentingNewLine]", ")"}]}], "\[IndentingNewLine]", 
  "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{"updates", " ", "current", " ", "message"}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"updateMsg", "[", 
     RowBox[{"{", 
      RowBox[{"i_", ",", "j_"}], "}"}], "]"}], ":=", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Assert", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"i", ",", "j"}], "}"}], "\[Element]", "messages"}], "]"}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"xs", ",", "ys"}], "}"}], "=", 
       RowBox[{"evalMsg", "[", 
        RowBox[{"{", 
         RowBox[{"i", ",", "j"}], "}"}], "]"}]}], ";", "\[IndentingNewLine]", 
      
      RowBox[{"setMsg", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"i", ",", "j"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"xs", ",", "ys"}], "}"}]}], "]"}], ";"}], "\n", ")"}]}], 
   ";"}], "\[IndentingNewLine]", "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{"RHS", " ", "of", " ", "message", " ", "equation"}], " ", 
   "*)"}]}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"evalMsg", "[", 
     RowBox[{"{", 
      RowBox[{"i_", ",", "j_"}], "}"}], "]"}], ":=", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Assert", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"i", ",", "j"}], "}"}], "\[Element]", "messages"}], "]"}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{"factors", "=", 
       RowBox[{
        RowBox[{"cl", "[", "j", "]"}], "\[Tilde]", 
        RowBox[{"ch", "[", 
         RowBox[{"{", 
          RowBox[{"i", ",", "j"}], "}"}], "]"}]}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"xsT", ",", "ysT"}], "}"}], "=", 
       RowBox[{"evalProduct", "[", 
        RowBox[{"factors", ",", "j"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"xsS", ",", "ysS"}], "}"}], "=", 
       RowBox[{"evalSum", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"xsT", ",", "ysT"}], "}"}], ",", 
         RowBox[{"i", "\[Intersection]", "j"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"normalizeTable", "[", 
       RowBox[{"{", 
        RowBox[{"xsS", ",", "ysS"}], "}"}], "]"}]}], "\n", ")"}]}], ";"}], 
  "\[IndentingNewLine]", "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{"LHS", " ", "of", " ", "message", " ", "equation"}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"evalCurrentMsg", "[", "msg_", "]"}], ":=", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Assert", "[", 
       RowBox[{"msg", "\[Element]", "messages"}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"cX", "[", "msg", "]"}], ",", 
        RowBox[{"cY", "[", "msg", "]"}]}], "}"}]}], "\[IndentingNewLine]", 
     ")"}]}], ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"evalClique", "[", "j_", "]"}], ":=", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Assert", "[", 
       RowBox[{"j", "\[Element]", "jnodes"}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"factors", "=", 
       RowBox[{
        RowBox[{"cl", "[", "j", "]"}], "\[Tilde]", 
        RowBox[{"ch", "[", 
         RowBox[{"{", "j", "}"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"xs", ",", "ys"}], "}"}], "=", 
       RowBox[{"normalizeTable", "@", 
        RowBox[{"evalProduct", "[", 
         RowBox[{"factors", ",", "j"}], "]"}]}]}]}], "\n", ")"}]}], ";"}], 
  "\[IndentingNewLine]", "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{"evaluates", " ", "separator", " ", "potential"}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"evalEdge", "[", "edge_", "]"}], ":=", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"prodMsg", "=", 
       RowBox[{"multiplyTables", "[", 
        RowBox[{
         RowBox[{"evalCurrentMsg", "[", "edge", "]"}], ",", 
         RowBox[{"evalCurrentMsg", "[", 
          RowBox[{"Reverse", "@", "edge"}], "]"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"normalizeTable", "@", "prodMsg"}]}], "\[IndentingNewLine]", 
     ")"}]}], ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"setMsg", "[", 
     RowBox[{"msg_", ",", 
      RowBox[{"{", 
       RowBox[{"xs_", ",", "ys_"}], "}"}]}], "]"}], ":=", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Assert", "[", 
       RowBox[{
        RowBox[{"msg", "\[Element]", "messages"}], "&&", 
        RowBox[{"isProbTable", "[", 
         RowBox[{"{", 
          RowBox[{"xs", ",", "ys"}], "}"}], "]"}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"cX", "[", "msg", "]"}], "=", "xs"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"cY", "[", "msg", "]"}], "=", "ys"}], ";"}], "\n", ")"}]}], 
   ";"}], "\[IndentingNewLine]", "\n", 
  RowBox[{"(*", " ", 
   RowBox[{
   "evaluates", " ", "product", " ", "of", " ", "factors", " ", "on", " ", 
    "given", " ", "domain"}], " ", "*)"}]}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"evalProduct", "[", 
     RowBox[{"factors_", ",", "domain_"}], "]"}], ":=", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Assert", "[", 
       RowBox[{
        RowBox[{"Union", "[", 
         RowBox[{"Flatten", "[", 
          RowBox[{"cD", "/@", "factors"}], "]"}], "]"}], "\[Subset]", 
        "domain"}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"formulas", "=", 
       RowBox[{"getFormula", "/@", "factors"}]}], ";", 
      RowBox[{"instances", "=", 
       RowBox[{"SatisfiabilityInstances", "[", 
        RowBox[{
         RowBox[{"And", "@@", "formulas"}], ",", "domain", ",", "All"}], 
        "]"}]}], ";", 
      RowBox[{"xs", "=", 
       RowBox[{"Sort", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"Extract", "[", 
           RowBox[{"domain", ",", 
            RowBox[{"Position", "[", 
             RowBox[{"#", ",", "True"}], "]"}]}], "]"}], "&"}], "/@", 
         "instances"}], "]"}]}], ";", 
      RowBox[{
       RowBox[{"oneVal", "[", "x_", "]"}], ":=", 
       RowBox[{"prod", "[", 
        RowBox[{
         RowBox[{"f", "\[Element]", "factors"}], ",", 
         RowBox[{"evalF", "[", 
          RowBox[{"f", ",", "x"}], "]"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"ys", "=", 
       RowBox[{"oneVal", "/@", "xs"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"xs", ",", "ys"}], "}"}]}], "\n", ")"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"getFormula", "[", "factor_List", "]"}], ":=", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Assert", "[", 
       RowBox[{"indexQ", "[", 
        RowBox[{"cX", ",", "factor"}], "]"}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Or", "@@", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"compact2full", "[", 
           RowBox[{"#", ",", 
            RowBox[{"cD", "[", "factor", "]"}]}], "]"}], "&"}], "/@", 
         RowBox[{"cX", "[", "factor", "]"}]}], ")"}]}]}], "\n", ")"}]}], 
   ";"}], "\[IndentingNewLine]", "\n", 
  RowBox[{"(*", " ", 
   RowBox[{
   "sums", " ", "over", " ", "atoms", " ", "orthogonal", " ", "to", " ", 
    "domain"}], " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"evalSum", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", "}"}], ",", 
       RowBox[{"{", "}"}]}], "}"}], ",", "_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", "}"}], ",", 
     RowBox[{"{", "}"}]}], "}"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"evalSum", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"xs_", ",", "ys_"}], "}"}], ",", "dom_"}], "]"}], ":=", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Assert", "[", 
       RowBox[{
        RowBox[{"isUnnormProbTable", "[", 
         RowBox[{"{", 
          RowBox[{"xs", ",", "ys"}], "}"}], "]"}], "&&", 
        RowBox[{"dom", "\[Subset]", 
         RowBox[{"Union", "[", 
          RowBox[{"Flatten", "[", "xs", "]"}], "]"}]}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"decorated", "=", 
       RowBox[{"Transpose", "[", 
        RowBox[{"{", 
         RowBox[{"xs", ",", "ys"}], "}"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"grouped", "=", 
       RowBox[{"GatherBy", "[", 
        RowBox[{"decorated", ",", 
         RowBox[{
          RowBox[{
           RowBox[{"First", "[", "#", "]"}], "\[Intersection]", "dom"}], 
          "&"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"getXY", "[", "group_", "]"}], ":=", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"x", "=", 
          RowBox[{
           RowBox[{"group", "[", 
            RowBox[{"[", 
             RowBox[{"1", ",", "1"}], "]"}], "]"}], "\[Intersection]", 
           "dom"}]}], ";", 
         RowBox[{"y", "=", 
          RowBox[{"Total", "@", 
           RowBox[{"group", "[", 
            RowBox[{"[", 
             RowBox[{"All", ",", "2"}], "]"}], "]"}]}]}], ";", 
         RowBox[{"{", 
          RowBox[{"x", ",", "y"}], "}"}]}], ")"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Transpose", "[", 
       RowBox[{"getXY", "/@", "grouped"}], "]"}]}], "\n", ")"}]}], ";"}], 
  "\n", "\n", 
  RowBox[{"(*", " ", 
   RowBox[{
   "looks", " ", "up", " ", "a", " ", "single", " ", "value", " ", "of", " ", 
    "a", " ", "factor"}], " ", "*)"}]}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"evalF", "[", 
     RowBox[{"factor_", ",", "rawX_"}], "]"}], ":=", 
    RowBox[{"(", "\n", "\t", 
     RowBox[{
      RowBox[{"Assert", "[", 
       RowBox[{"indexQ", "[", 
        RowBox[{"cX", ",", "factor"}], "]"}], "]"}], ";", "\n", "\t", 
      RowBox[{"x", "=", 
       RowBox[{"rawX", "\[Intersection]", 
        RowBox[{"cD", "[", "factor", "]"}]}]}], ";", "\n", "\t", 
      RowBox[{"Assert", "[", 
       RowBox[{"MemberQ", "[", 
        RowBox[{
         RowBox[{"cX", "[", "factor", "]"}], ",", "x"}], "]"}], "]"}], ";", 
      "\n", "\t", 
      RowBox[{"vals", "=", 
       RowBox[{"Extract", "[", 
        RowBox[{
         RowBox[{"cY", "[", "factor", "]"}], ",", 
         RowBox[{"Position", "[", 
          RowBox[{
           RowBox[{"cX", "[", "factor", "]"}], ",", "x"}], "]"}]}], "]"}]}], 
      ";", "\n", "\t", 
      RowBox[{"Assert", "[", 
       RowBox[{
        RowBox[{"Length", "[", "vals", "]"}], "==", "1"}], "]"}], ";", "\n", 
      "\t", 
      RowBox[{"First", "@", "vals"}]}], "\n", ")"}]}], ";"}], 
  "\[IndentingNewLine]", "\[IndentingNewLine]", "\[IndentingNewLine]", 
  "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
   "finds", " ", "discrepancy", " ", "on", " ", "separator", " ", 
    "marginal"}], " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"getMiscalibration", "[", "edge_", "]"}], ":=", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Assert", "[", 
       RowBox[{"edge", "\[Element]", "jedges"}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"sep", "=", 
       RowBox[{"Intersection", "@@", "edge"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"cliques", "=", 
       RowBox[{"Select", "[", 
        RowBox[{"jnodes", ",", 
         RowBox[{
          RowBox[{"sep", "\[Subset]", "#"}], "&"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"distrs", "=", 
       RowBox[{"evalClique", "/@", "edge"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"marg1", ",", "marg2"}], "}"}], "=", 
       RowBox[{
        RowBox[{
         RowBox[{"evalSum", "[", 
          RowBox[{"#", ",", "sep"}], "]"}], "&"}], "/@", "distrs"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"projDist", "[", 
       RowBox[{
        RowBox[{"getYS", "@", "marg1"}], ",", 
        RowBox[{"getYS", "@", "marg2"}]}], "]"}]}], "\[IndentingNewLine]", 
     ")"}]}], ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"getMaxMiscalibration", "[", "]"}], ":=", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{"Max", "[", 
      RowBox[{"getMiscalibration", "/@", 
       RowBox[{"Select", "[", 
        RowBox[{"jedges", ",", "OrderedQ"}], "]"}]}], "]"}], 
     "\[IndentingNewLine]", ")"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"getResidual", "[", "msg_", "]"}], ":=", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Assert", "[", 
       RowBox[{"msg", "\[Element]", "messages"}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"oldMsg", "=", 
       RowBox[{"evalCurrentMsg", "[", "msg", "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"newMsg", "=", 
       RowBox[{"evalMsg", "[", "msg", "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"msgDist", "[", 
       RowBox[{"oldMsg", ",", 
        RowBox[{"cD", "[", "msg", "]"}], ",", "newMsg", ",", 
        RowBox[{"cD", "[", "msg", "]"}]}], "]"}]}], "\[IndentingNewLine]", 
     ")"}]}], ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"getTotalResidual", "[", "]"}], ":=", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{"Total", "[", 
      RowBox[{"getResidual", "/@", "messages"}], "]"}], "\[IndentingNewLine]",
      ")"}]}], ";"}], "\[IndentingNewLine]", 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"cliqueAveEnergy", "[", "clique_", "]"}], ":=", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Assert", "[", 
       RowBox[{"clique", "\[Element]", "jnodes"}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"factors", "=", 
       RowBox[{"cl", "[", "clique", "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"cliquePot", "=", 
       RowBox[{"compact2fullTable", "[", 
        RowBox[{
         RowBox[{"evalProduct", "[", 
          RowBox[{"factors", ",", "clique"}], "]"}], ",", "clique"}], "]"}]}],
       ";", "\[IndentingNewLine]", 
      RowBox[{"cliqueProb", "=", 
       RowBox[{"compact2fullTable", "[", 
        RowBox[{
         RowBox[{"evalClique", "[", "clique", "]"}], ",", "clique"}], "]"}]}],
       ";", "\[IndentingNewLine]", 
      RowBox[{"expectedEnergy", "[", 
       RowBox[{
        RowBox[{"cliqueProb", "[", 
         RowBox[{"[", "2", "]"}], "]"}], ",", 
        RowBox[{"cliquePot", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}], "]"}]}], "\[IndentingNewLine]", 
     ")"}]}], ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"cliqueEntropy", "[", "clique_", "]"}], ":=", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Assert", "[", 
       RowBox[{"clique", "\[Element]", "jnodes"}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"xs", ",", "ys"}], "}"}], "=", 
       RowBox[{"evalClique", "[", "clique", "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"entropy", "[", "ys", "]"}]}], "\[IndentingNewLine]", ")"}]}], 
   ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"edgeEntropy", "[", "edge_", "]"}], ":=", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Assert", "[", 
       RowBox[{"edge", "\[Element]", "jedges"}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"xs", ",", "ys"}], "}"}], "=", 
       RowBox[{"evalEdge", "[", "edge", "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"entropy", "[", "ys", "]"}]}], "\[IndentingNewLine]", ")"}]}], 
   ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"betheFreeEnergy", "[", "]"}], ":=", 
   RowBox[{"(", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Total", "[", 
      RowBox[{"cliqueAveEnergy", "/@", "jnodes"}], "]"}], "+", 
     RowBox[{"Total", "[", 
      RowBox[{"cliqueEntropy", "/@", "jnodes"}], "]"}], "-", 
     RowBox[{"Total", "[", 
      RowBox[{"edgeEntropy", "/@", "jedges"}], "]"}]}], "\[IndentingNewLine]",
     ")"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.5058471363901987`*^9, 3.505847152721148*^9}, {
   3.505847197248345*^9, 3.505847379792232*^9}, {3.505847434260853*^9, 
   3.50584743456964*^9}, {3.505847666683743*^9, 3.505847676865614*^9}, {
   3.505848547873954*^9, 3.505848558327177*^9}, {3.505848592172024*^9, 
   3.505848593778191*^9}, {3.505848626080451*^9, 3.505848628409343*^9}, {
   3.505848793490781*^9, 3.505848832656464*^9}, 3.505848889141788*^9, {
   3.505849596234263*^9, 3.5058496268771133`*^9}, {3.505851892951354*^9, 
   3.505851904721637*^9}, {3.505851960101824*^9, 3.505851966615746*^9}, {
   3.5058520035370903`*^9, 3.505852096002619*^9}, {3.5058532815803556`*^9, 
   3.505853310753735*^9}, 3.5058534866039543`*^9, {3.505854792595587*^9, 
   3.505854792979125*^9}, {3.505854864788084*^9, 3.505854883950026*^9}, {
   3.505855347744793*^9, 3.505855369364921*^9}, {3.505877435243672*^9, 
   3.505877510923278*^9}, 3.505877664522026*^9, {3.505877711590891*^9, 
   3.505877718006159*^9}, {3.505930403578116*^9, 3.505930409935217*^9}, {
   3.5059336403123007`*^9, 3.50593372095116*^9}, {3.5059338340477543`*^9, 
   3.505933848308804*^9}, {3.5059344666213703`*^9, 3.505934466957314*^9}, {
   3.505935817033087*^9, 3.505935819671525*^9}, {3.505946346685872*^9, 
   3.505946368464951*^9}, {3.505946425888102*^9, 3.505946428415728*^9}, {
   3.5059498646883783`*^9, 3.5059498650512447`*^9}, {3.505950222213972*^9, 
   3.505950272733289*^9}, {3.5059514915622807`*^9, 3.505951538410201*^9}, {
   3.505951804187439*^9, 3.505951805146639*^9}, {3.5059564731499453`*^9, 
   3.505956482820067*^9}, {3.505961269364064*^9, 3.505961327693108*^9}, {
   3.505963180750781*^9, 3.5059631940444717`*^9}, {3.505963937498198*^9, 
   3.5059639505425577`*^9}, {3.505966812114092*^9, 3.505966828482732*^9}, {
   3.5059670344084463`*^9, 3.505967051050445*^9}, {3.5059671015983152`*^9, 
   3.505967154379608*^9}, {3.505967268543129*^9, 3.505967268930414*^9}, {
   3.505967826351439*^9, 3.505967835766938*^9}, {3.505967994458432*^9, 
   3.505968024868738*^9}, {3.505968206146266*^9, 3.505968304620944*^9}, {
   3.5059684593241243`*^9, 3.505968481424855*^9}, {3.5059685181347637`*^9, 
   3.505968534499383*^9}, {3.505968575545659*^9, 3.505968644391239*^9}, {
   3.505968679622919*^9, 3.505968687360367*^9}, {3.505969096439266*^9, 
   3.505969254979199*^9}, {3.505969297024358*^9, 3.505969389969823*^9}, {
   3.505969422767963*^9, 3.505969444510439*^9}, {3.505969491439328*^9, 
   3.5059695006752357`*^9}, {3.505969545120241*^9, 3.505969570209606*^9}, {
   3.5059696093508387`*^9, 3.505969610171191*^9}, {3.505969781931139*^9, 
   3.505969793925856*^9}, {3.505969939338331*^9, 3.5059699486410923`*^9}, {
   3.505970204706612*^9, 3.5059702054683743`*^9}, 3.505970268682757*^9, {
   3.506004325444314*^9, 3.5060043405548363`*^9}, {3.5060045021068077`*^9, 
   3.506004505632897*^9}, {3.5060053503634872`*^9, 3.506005352354693*^9}, 
   3.5060101171564283`*^9, {3.506010172287469*^9, 3.5060101815750227`*^9}, {
   3.506010331809457*^9, 3.506010332446801*^9}, {3.5060103715169773`*^9, 
   3.506010379715478*^9}, {3.5060104502944508`*^9, 3.506010452331498*^9}, {
   3.506010502173812*^9, 3.5060105406093407`*^9}, {3.50601063069278*^9, 
   3.506010646441533*^9}, {3.506010913123513*^9, 3.506010923993042*^9}, {
   3.5060115820099583`*^9, 3.5060116232140617`*^9}, {3.50601171554399*^9, 
   3.506011740493989*^9}, {3.506011791337063*^9, 3.506011801526854*^9}, {
   3.506042813981359*^9, 3.5060428246684637`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Usage", "Section",
 CellChangeTimes->{{3.505854738887496*^9, 3.5058547415374193`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"initHardcore", "@", 
   RowBox[{"PathGraph", "@", 
    RowBox[{"Range", "@", "3"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"decomp", "=", 
   RowBox[{"findTreeDecomposition", "@", "g"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"initMessages", "@", "decomp"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Lambda]", "=", "1."}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"parallelUpdate", "[", "10", "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"evalClique", "[", 
  RowBox[{"First", "[", "jnodes", "]"}], "]"}]}], "Input",
 CellChangeTimes->{{3.505854749108946*^9, 3.505854756484585*^9}, {
  3.5060422872507353`*^9, 3.506042292638801*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", "}"}], ",", 
     RowBox[{"{", "1", "}"}], ",", 
     RowBox[{"{", "2", "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.4`", ",", "0.4`", ",", "0.2`"}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.506042292945174*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"initHardcore", "@", 
   RowBox[{"CycleGraph", "[", "3", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"\[Lambda]", "=", "1."}], ";"}], "\[IndentingNewLine]", 
  RowBox[{"(*", 
   RowBox[{
    RowBox[{"decomp", "=", 
     RowBox[{"findTreeDecomposition", "@", "g"}]}], ";"}], 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{"decomp", "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"1", ",", "2"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"1", ",", "3"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"2", ",", "3"}], "}"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"1", ",", "2"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"2", ",", "3"}], "}"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"1", ",", "2"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"1", ",", "3"}], "}"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"1", ",", "3"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"2", ",", "3"}], "}"}]}], "}"}]}], "}"}]}], 
   "}"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"initMessages", "@", "decomp"}], ";"}]}], "Input",
 CellChangeTimes->{{3.505935203887755*^9, 3.5059352051336403`*^9}, {
  3.505950028875615*^9, 3.505950036542633*^9}, {3.5059521915072527`*^9, 
  3.50595219652914*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", "2"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "3"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"2", ",", "3"}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"1", ",", "2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "3"}], "}"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"1", ",", "2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "3"}], "}"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"1", ",", "3"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "3"}], "}"}]}], "}"}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{{3.505935198685895*^9, 3.505935205460991*^9}, 
   3.5059352823005257`*^9, 3.505949937552064*^9, 3.505950036991601*^9, 
   3.505952149571637*^9, 3.5059557031112413`*^9, 3.5060422988730593`*^9}]
}, Open  ]],

Cell[BoxData[{
 RowBox[{
  RowBox[{"g", "=", 
   RowBox[{"PathGraph", "[", 
    RowBox[{"Range", "[", "3", "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"initHardcore", "@", "g"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Lambda]", "=", "1."}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"decomp", "=", 
   RowBox[{"findTreeDecomposition", "@", "g"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"initMessages", "@", "decomp"}], ";"}]}], "Input",
 CellChangeTimes->{{3.505952205934614*^9, 3.505952246144463*^9}, {
  3.505956023690181*^9, 3.505956024198862*^9}, {3.505956087739993*^9, 
  3.505956090507073*^9}, {3.505956419394945*^9, 3.505956420967422*^9}, {
  3.505956546218004*^9, 3.505956559951577*^9}, {3.505956666506158*^9, 
  3.505956666566395*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Test", "Section",
 CellChangeTimes->{{3.506018803038625*^9, 3.506018806244585*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"gname", "=", 
   RowBox[{"{", 
    RowBox[{"\"\<Cubic\>\"", ",", 
     RowBox[{"{", 
      RowBox[{"10", ",", "3"}], "}"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Lambda]", "=", "1."}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"g", "=", 
   RowBox[{"GraphData", "[", "gname", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"initHardcore", "@", "g"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"decomp", "=", 
   RowBox[{"findTreeDecomposition", "@", "g"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"initMessages", "@", "decomp"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"diam", "=", 
   RowBox[{"GraphDiameter", "[", 
    RowBox[{"Graph", "[", 
     RowBox[{"UndirectedEdge", "@@@", 
      RowBox[{"Last", "@", "decomp"}]}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"parallelUpdate", "[", "diam", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"getMaxMiscalibration", "[", "]"}], "\[IndentingNewLine]", 
 RowBox[{"getTotalResidual", "[", "]"}], "\[IndentingNewLine]", 
 RowBox[{"Exp", "[", 
  RowBox[{"betheFreeEnergy", "[", "]"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"GraphData", "[", 
   RowBox[{"gname", ",", "\"\<IndependencePolynomial\>\""}], "]"}], "[", "1", 
  "]"}]}], "Input",
 CellChangeTimes->{{3.506018817326239*^9, 3.5060188176058607`*^9}, {
  3.506018930083807*^9, 3.506018941331132*^9}, {3.5060523137578573`*^9, 
  3.506052321987978*^9}}],

Cell[BoxData["4.440892098500625`*^-16"], "Output",
 CellChangeTimes->{
  3.506018818793785*^9, {3.5060189356025248`*^9, 3.506018942780081*^9}, 
   3.506042312133741*^9, 3.506052278707645*^9, 3.506052322612235*^9}],

Cell[BoxData["0.`"], "Output",
 CellChangeTimes->{
  3.506018818793785*^9, {3.5060189356025248`*^9, 3.506018942780081*^9}, 
   3.506042312133741*^9, 3.506052278707645*^9, 3.5060523227341537`*^9}],

Cell[BoxData["71.99999999999994`"], "Output",
 CellChangeTimes->{
  3.506018818793785*^9, {3.5060189356025248`*^9, 3.506018942780081*^9}, 
   3.506042312133741*^9, 3.506052278707645*^9, 3.506052322827372*^9}],

Cell[BoxData["72"], "Output",
 CellChangeTimes->{
  3.506018818793785*^9, {3.5060189356025248`*^9, 3.506018942780081*^9}, 
   3.506042312133741*^9, 3.506052278707645*^9, 3.506052322854364*^9}]
}, Open  ]],

Cell[BoxData[{
 RowBox[{
  RowBox[{"gname", "=", 
   RowBox[{"{", 
    RowBox[{"\"\<Cubic\>\"", ",", 
     RowBox[{"{", 
      RowBox[{"10", ",", "3"}], "}"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"g", "=", 
   RowBox[{"GraphData", "[", "gname", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"initHardcore", "@", "g"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"decompBethe", "=", 
   RowBox[{"{", 
    RowBox[{"edges", ",", 
     RowBox[{"intersectingPairs", "[", "edges", "]"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"initMessages", "@", "decomp"}], 
  ";"}], "\[IndentingNewLine]"}], "Input",
 CellChangeTimes->{
  3.50604339989355*^9, {3.506043447598322*^9, 3.506043447988908*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Test2", "Section",
 CellChangeTimes->{{3.506051775262979*^9, 3.5060517779507627`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"gname", "=", 
   RowBox[{"{", 
    RowBox[{"\"\<Fan\>\"", ",", 
     RowBox[{"{", 
      RowBox[{"3", ",", "2"}], "}"}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"g", "=", 
   RowBox[{"GraphData", "[", "gname", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"initHardcore", "@", "g"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"decompBethe", "=", 
   RowBox[{"{", 
    RowBox[{"edges", ",", 
     RowBox[{"intersectingPairs", "[", "edges", "]"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"initMessages", "@", "decompBethe"}], ";"}]}], "Input",
 CellChangeTimes->{{3.506051807666128*^9, 3.506051808481876*^9}, {
  3.506051878575947*^9, 3.506051879104459*^9}, {3.506052034612494*^9, 
  3.506052035197514*^9}, {3.506052432580811*^9, 3.506052435039143*^9}, {
  3.50605252238958*^9, 3.5060525226512136`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"oneStep", "[", "]"}], ":=", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"pic2", "=", 
       RowBox[{"visualizeCavities", "[", "]"}]}], ";", "\[IndentingNewLine]", 
      
      RowBox[{"parallelUpdate", "[", "1", "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"Show", "[", "pic2", "]"}]}], "\[IndentingNewLine]", ")"}]}], 
   ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{"pics", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"oneStep", "[", "]"}], "&"}], "/@", 
   RowBox[{"Range", "[", "5", "]"}]}]}]}], "Input",
 CellChangeTimes->{{3.506052437827587*^9, 3.506052444086626*^9}, {
  3.506052511073118*^9, 3.506052529229033*^9}, {3.50605258235063*^9, 
  3.5060525827643547`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   GraphicsBox[{GraphicsGroupBox[{
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[1.1195027337773278`*^-15], 
         DiskBox[{0.587785, -0.809017}, 0.23511400000000002`, {
          0, 3.141592653589793}], 
         {Opacity[4.447455070709941*^-31], 
          DiskBox[{0.587785, -0.809017}, 0.23511400000000002`, {
           3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[1.1195027337773282`*^-15], 
         DiskBox[{-0.587785, -0.809017}, 0.23511400000000002`, {
          0, 3.141592653589793}], 
         {Opacity[4.447455070709941*^-31], 
          DiskBox[{-0.587785, -0.809017}, 0.23511400000000002`, {
           3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.4999999999920054], 
         DiskBox[{-0.951057, 0.309017}, 0.23511400000000002`, {
          0, 3.141592653589793}], 
         {Opacity[0.5], 
          DiskBox[{-0.951057, 0.309017}, 0.23511400000000002`, {
           3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.4999999999920054], 
         DiskBox[{0., 1.}, 0.23511400000000002`, {0, 3.141592653589793}], 
         {Opacity[0.5], 
          DiskBox[{0., 1.}, 0.23511400000000002`, {3.141592653589793, 
           6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.4999999999920054], 
         DiskBox[{0.951057, 0.309017}, 0.23511400000000002`, {
          0, 3.141592653589793}], 
         {Opacity[0.5], 
          DiskBox[{0.951057, 0.309017}, 0.23511400000000002`, {
           3.141592653589793, 6.283185307179586}]}}}, 
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        LineBox[{{{
         0.34293620940405223`, -0.926574}, {-0.3429362094040528, -0.926574}}, \
{{0.42337943742201406`, -0.5971313039341559}, {-0.7866514374220144, 
         0.09713130393415638}}, {{0.5411184140732639, -0.5665015890336643}, {
         0.046666585926735706`, 0.7574845890336646}}, {{
         0.6389998925370635, -0.566983979500693}, {0.8998421074629361, 
         0.06698397950069344}}, {{-0.6389998925370637, -0.566983979500693}, \
{-0.8998421074629365, 
         0.06698397950069344}}, {{-0.5411184140732642, -0.5665015890336643}, \
{-0.04666658592673614, 
         0.7574845890336644}}, {{-0.42337943742201445`, -0.597131303934156}, {
         0.7866514374220139, 0.09713130393415638}}}]}}], {}},
    FrameTicks->None], ",", 
   GraphicsBox[{GraphicsGroupBox[{
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[4.447455070709941*^-31], 
         DiskBox[{0.587785, -0.809017}, 0.23511400000000002`, {
          0, 3.141592653589793}], 
         {Opacity[8.370761074590194*^-62], 
          DiskBox[{0.587785, -0.809017}, 0.23511400000000002`, {
           3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[4.447455070709941*^-31], 
         DiskBox[{-0.587785, -0.809017}, 0.23511400000000002`, {
          0, 3.141592653589793}], 
         {Opacity[8.370761074590194*^-62], 
          DiskBox[{-0.587785, -0.809017}, 0.23511400000000002`, {
           3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.5], 
         DiskBox[{-0.951057, 0.309017}, 0.23511400000000002`, {
          0, 3.141592653589793}], 
         {Opacity[0.5], 
          DiskBox[{-0.951057, 0.309017}, 0.23511400000000002`, {
           3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.5], 
         DiskBox[{0., 1.}, 0.23511400000000002`, {0, 3.141592653589793}], 
         {Opacity[0.5], 
          DiskBox[{0., 1.}, 0.23511400000000002`, {3.141592653589793, 
           6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.5], 
         DiskBox[{0.951057, 0.309017}, 0.23511400000000002`, {
          0, 3.141592653589793}], 
         {Opacity[0.5], 
          DiskBox[{0.951057, 0.309017}, 0.23511400000000002`, {
           3.141592653589793, 6.283185307179586}]}}}, 
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        LineBox[{{{
         0.34293620940405223`, -0.926574}, {-0.3429362094040528, -0.926574}}, \
{{0.42337943742201406`, -0.5971313039341559}, {-0.7866514374220144, 
         0.09713130393415638}}, {{0.5411184140732639, -0.5665015890336643}, {
         0.046666585926735706`, 0.7574845890336646}}, {{
         0.6389998925370635, -0.566983979500693}, {0.8998421074629361, 
         0.06698397950069344}}, {{-0.6389998925370637, -0.566983979500693}, \
{-0.8998421074629365, 
         0.06698397950069344}}, {{-0.5411184140732642, -0.5665015890336643}, \
{-0.04666658592673614, 
         0.7574845890336644}}, {{-0.42337943742201445`, -0.597131303934156}, {
         0.7866514374220139, 0.09713130393415638}}}]}}], {}},
    FrameTicks->None], ",", 
   GraphicsBox[{GraphicsGroupBox[{
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[8.370761074590194*^-62], 
         DiskBox[{0.587785, -0.809017}, 0.23511400000000002`, {
          0, 3.141592653589793}], 
         {Opacity[1.387050587093738*^-123], 
          DiskBox[{0.587785, -0.809017}, 0.23511400000000002`, {
           3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[8.370761074590194*^-62], 
         DiskBox[{-0.587785, -0.809017}, 0.23511400000000002`, {
          0, 3.141592653589793}], 
         {Opacity[1.387050587093738*^-123], 
          DiskBox[{-0.587785, -0.809017}, 0.23511400000000002`, {
           3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.5], 
         DiskBox[{-0.951057, 0.309017}, 0.23511400000000002`, {
          0, 3.141592653589793}], 
         {Opacity[0.5], 
          DiskBox[{-0.951057, 0.309017}, 0.23511400000000002`, {
           3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.5], 
         DiskBox[{0., 1.}, 0.23511400000000002`, {0, 3.141592653589793}], 
         {Opacity[0.5], 
          DiskBox[{0., 1.}, 0.23511400000000002`, {3.141592653589793, 
           6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.5], 
         DiskBox[{0.951057, 0.309017}, 0.23511400000000002`, {
          0, 3.141592653589793}], 
         {Opacity[0.5], 
          DiskBox[{0.951057, 0.309017}, 0.23511400000000002`, {
           3.141592653589793, 6.283185307179586}]}}}, 
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        LineBox[{{{
         0.34293620940405223`, -0.926574}, {-0.3429362094040528, -0.926574}}, \
{{0.42337943742201406`, -0.5971313039341559}, {-0.7866514374220144, 
         0.09713130393415638}}, {{0.5411184140732639, -0.5665015890336643}, {
         0.046666585926735706`, 0.7574845890336646}}, {{
         0.6389998925370635, -0.566983979500693}, {0.8998421074629361, 
         0.06698397950069344}}, {{-0.6389998925370637, -0.566983979500693}, \
{-0.8998421074629365, 
         0.06698397950069344}}, {{-0.5411184140732642, -0.5665015890336643}, \
{-0.04666658592673614, 
         0.7574845890336644}}, {{-0.42337943742201445`, -0.597131303934156}, {
         0.7866514374220139, 0.09713130393415638}}}]}}], {}},
    FrameTicks->None], ",", 
   GraphicsBox[{GraphicsGroupBox[{
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[1.387050587093738*^-123], 
         DiskBox[{0.587785, -0.809017}, 0.23511400000000002`, {
          0, 3.141592653589793}], 
         {Opacity[1.5565734905400044`*^-246], 
          DiskBox[{0.587785, -0.809017}, 0.23511400000000002`, {
           3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[1.387050587093738*^-123], 
         DiskBox[{-0.587785, -0.809017}, 0.23511400000000002`, {
          0, 3.141592653589793}], 
         {Opacity[1.556573490540005*^-246], 
          DiskBox[{-0.587785, -0.809017}, 0.23511400000000002`, {
           3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.5], 
         DiskBox[{-0.951057, 0.309017}, 0.23511400000000002`, {
          0, 3.141592653589793}], 
         {Opacity[0.5], 
          DiskBox[{-0.951057, 0.309017}, 0.23511400000000002`, {
           3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.5], 
         DiskBox[{0., 1.}, 0.23511400000000002`, {0, 3.141592653589793}], 
         {Opacity[0.5], 
          DiskBox[{0., 1.}, 0.23511400000000002`, {3.141592653589793, 
           6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.5], 
         DiskBox[{0.951057, 0.309017}, 0.23511400000000002`, {
          0, 3.141592653589793}], 
         {Opacity[0.5], 
          DiskBox[{0.951057, 0.309017}, 0.23511400000000002`, {
           3.141592653589793, 6.283185307179586}]}}}, 
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        LineBox[{{{
         0.34293620940405223`, -0.926574}, {-0.3429362094040528, -0.926574}}, \
{{0.42337943742201406`, -0.5971313039341559}, {-0.7866514374220144, 
         0.09713130393415638}}, {{0.5411184140732639, -0.5665015890336643}, {
         0.046666585926735706`, 0.7574845890336646}}, {{
         0.6389998925370635, -0.566983979500693}, {0.8998421074629361, 
         0.06698397950069344}}, {{-0.6389998925370637, -0.566983979500693}, \
{-0.8998421074629365, 
         0.06698397950069344}}, {{-0.5411184140732642, -0.5665015890336643}, \
{-0.04666658592673614, 
         0.7574845890336644}}, {{-0.42337943742201445`, -0.597131303934156}, {
         0.7866514374220139, 0.09713130393415638}}}]}}], {}},
    FrameTicks->None], ",", 
   GraphicsBox[{GraphicsGroupBox[{
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[1.5565734905400044`*^-246], 
         DiskBox[{0.587785, -0.809017}, 0.23511400000000002`, {
          0, 3.141592653589793}], 
         {Opacity[
          4.6258655574323402350205872089704707838462361953`15.954589770191005*\
^-493], DiskBox[{0.587785, -0.809017}, 0.23511400000000002`, {
           3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[1.556573490540005*^-246], 
         DiskBox[{-0.587785, -0.809017}, 0.23511400000000002`, {
          0, 3.141592653589793}], 
         {Opacity[
          4.6258655574323367485832391516915328209390665029`15.954589770191005*\
^-493], DiskBox[{-0.587785, -0.809017}, 0.23511400000000002`, {
           3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.5], 
         DiskBox[{-0.951057, 0.309017}, 0.23511400000000002`, {
          0, 3.141592653589793}], 
         {Opacity[0.5], 
          DiskBox[{-0.951057, 0.309017}, 0.23511400000000002`, {
           3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.5], 
         DiskBox[{0., 1.}, 0.23511400000000002`, {0, 3.141592653589793}], 
         {Opacity[0.5], 
          DiskBox[{0., 1.}, 0.23511400000000002`, {3.141592653589793, 
           6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.5], 
         DiskBox[{0.951057, 0.309017}, 0.23511400000000002`, {
          0, 3.141592653589793}], 
         {Opacity[0.5], 
          DiskBox[{0.951057, 0.309017}, 0.23511400000000002`, {
           3.141592653589793, 6.283185307179586}]}}}, 
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        LineBox[{{{
         0.34293620940405223`, -0.926574}, {-0.3429362094040528, -0.926574}}, \
{{0.42337943742201406`, -0.5971313039341559}, {-0.7866514374220144, 
         0.09713130393415638}}, {{0.5411184140732639, -0.5665015890336643}, {
         0.046666585926735706`, 0.7574845890336646}}, {{
         0.6389998925370635, -0.566983979500693}, {0.8998421074629361, 
         0.06698397950069344}}, {{-0.6389998925370637, -0.566983979500693}, \
{-0.8998421074629365, 
         0.06698397950069344}}, {{-0.5411184140732642, -0.5665015890336643}, \
{-0.04666658592673614, 
         0.7574845890336644}}, {{-0.42337943742201445`, -0.597131303934156}, {
         0.7866514374220139, 0.09713130393415638}}}]}}], {}},
    FrameTicks->None]}], "}"}]], "Output",
 CellChangeTimes->{3.506052585194049*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"SeedRandom", "[", "3", "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gname", "=", 
   RowBox[{"{", 
    RowBox[{"7", ",", "446"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"g", "=", 
   RowBox[{"GraphData", "[", "gname", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"initHardcore", "@", "g"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"decompBethe", "=", 
   RowBox[{"{", 
    RowBox[{"edges", ",", 
     RowBox[{"intersectingPairs", "[", "edges", "]"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"initMessages", "@", "decompBethe"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"pics", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"oneStep", "[", "]"}], "&"}], "/@", 
   RowBox[{"Range", "[", "10", "]"}]}]}]}], "Input",
 CellChangeTimes->{{3.5060536012618017`*^9, 3.506053638172789*^9}, {
  3.5060536878988533`*^9, 3.5060536880007257`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   GraphicsBox[{GraphicsGroupBox[{
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.3418845944674212], 
         DiskBox[{2.341421002111228, 1.2622718318610946`}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[0.2730053864513298], 
          DiskBox[{2.341421002111228, 1.2622718318610946`}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.5591211414040042], 
         DiskBox[{0.4781404745760385, 0.8304391288619981}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[0.2899556353566147], 
          DiskBox[{0.4781404745760385, 0.8304391288619981}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.5930868414237174], 
         DiskBox[{1.3991853397812652`, 0.}, 0.14742528472252805`, {
          0, 3.141592653589793}], 
         {Opacity[0.36713920973845204`], 
          DiskBox[{1.3991853397812652`, 0.}, 0.14742528472252805`, {
           3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.5588481899413024], 
         DiskBox[{1.9202254717071263`, 1.7920472987842249`}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[0.43302521119637893`], 
          DiskBox[{1.9202254717071263`, 1.7920472987842249`}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.5167742963836354], 
         DiskBox[{0., 1.5294776837065591`}, 0.14742528472252805`, {
          0, 3.141592653589793}], 
         {Opacity[0.282764739254633], 
          DiskBox[{0., 1.5294776837065591`}, 0.14742528472252805`, {
           3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.629367530074202], 
         DiskBox[{1.6883121489539397`, 0.7643945280875081}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[0.004537345319965761], 
          DiskBox[{1.6883121489539397`, 0.7643945280875081}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.5356444893767885], 
         DiskBox[{1.1532983406755593`, 1.2143678573508532`}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[0.0026475177393801277`], 
          DiskBox[{1.1532983406755593`, 1.2143678573508532`}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}}, 
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], LineBox[CompressedData["
1:eJwBdQGK/iFib1JlAwAAAAsAAAACAAAAAgAAACquB4jeFgJAvSWX85iK9j/K
oDmT+wAAQKuNA8nkU/o/kAjWApbWAUBLN5w39g7yPwbYSnKdzPw/x8/D+Ya8
7D+faOpLEXgBQGU58HpH6vI/IquKPDz69D8aPtrXLlryPyumLe58WNo/sYW8
JjFj7z+7F3xMdgWxP2zwVY6fEPY/9kJ1mYpW5D/7c1skufDnPwevK+2Efvg/
rwh6OHJg5j8MNMCP9ojjP20GE9GKau4/tMluTuRV8D/bJj0fQoLxP0gEYRt2
APc/dZwzGGbuwz+flOio7WX6P+4qX/1ReuM/IkJwE94T9j/ibYxajizEP/3p
QQ0bw/I/UJ04Mnvo8D8cba8v6V7+P3/6meVwJ/o/m5ICwqhd+z+AV91LfX/t
P35R8M1I+vw/hpNEzIyD+j9yQSqA3jL0P8A0GPu5lvU/Qgar1EmLwz++Zdiu
4e/2P+pXZrt/AvA/inkp5uj29D/LArPg
         "]]}}], {}},
    FrameTicks->None], ",", 
   GraphicsBox[{GraphicsGroupBox[{
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.2730053864513298], 
         DiskBox[{2.341421002111228, 1.2622718318610946`}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[0.13662176173969293`], 
          DiskBox[{2.341421002111228, 1.2622718318610946`}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.2899556353566147], 
         DiskBox[{0.4781404745760385, 0.8304391288619981}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[0.14654692669928446`], 
          DiskBox[{0.4781404745760385, 0.8304391288619981}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.36713920973845204`], 
         DiskBox[{1.3991853397812652`, 0.}, 0.14742528472252805`, {
          0, 3.141592653589793}], 
         {Opacity[0.36990399808618973`], 
          DiskBox[{1.3991853397812652`, 0.}, 0.14742528472252805`, {
           3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.43302521119637893`], 
         DiskBox[{1.9202254717071263`, 1.7920472987842249`}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[0.4170674250719294], 
          DiskBox[{1.9202254717071263`, 1.7920472987842249`}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.282764739254633], 
         DiskBox[{0., 1.5294776837065591`}, 0.14742528472252805`, {
          0, 3.141592653589793}], 
         {Opacity[0.07162983824482305], 
          DiskBox[{0., 1.5294776837065591`}, 0.14742528472252805`, {
           3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.004537345319965761], 
         DiskBox[{1.6883121489539397`, 0.7643945280875081}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[0.4893507402506109], 
          DiskBox[{1.6883121489539397`, 0.7643945280875081}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.0026475177393801277`], 
         DiskBox[{1.1532983406755593`, 1.2143678573508532`}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[0.2893431770031411], 
          DiskBox[{1.1532983406755593`, 1.2143678573508532`}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}}, 
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], LineBox[CompressedData["
1:eJwBdQGK/iFib1JlAwAAAAsAAAACAAAAAgAAACquB4jeFgJAvSWX85iK9j/K
oDmT+wAAQKuNA8nkU/o/kAjWApbWAUBLN5w39g7yPwbYSnKdzPw/x8/D+Ya8
7D+faOpLEXgBQGU58HpH6vI/IquKPDz69D8aPtrXLlryPyumLe58WNo/sYW8
JjFj7z+7F3xMdgWxP2zwVY6fEPY/9kJ1mYpW5D/7c1skufDnPwevK+2Efvg/
rwh6OHJg5j8MNMCP9ojjP20GE9GKau4/tMluTuRV8D/bJj0fQoLxP0gEYRt2
APc/dZwzGGbuwz+flOio7WX6P+4qX/1ReuM/IkJwE94T9j/ibYxajizEP/3p
QQ0bw/I/UJ04Mnvo8D8cba8v6V7+P3/6meVwJ/o/m5ICwqhd+z+AV91LfX/t
P35R8M1I+vw/hpNEzIyD+j9yQSqA3jL0P8A0GPu5lvU/Qgar1EmLwz++Zdiu
4e/2P+pXZrt/AvA/inkp5uj29D/LArPg
         "]]}}], {}},
    FrameTicks->None], ",", 
   GraphicsBox[{GraphicsGroupBox[{
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.13662176173969293`], 
         DiskBox[{2.341421002111228, 1.2622718318610946`}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[0.0008139586421020535], 
          DiskBox[{2.341421002111228, 1.2622718318610946`}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.14654692669928446`], 
         DiskBox[{0.4781404745760385, 0.8304391288619981}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[0.00736591674056535], 
          DiskBox[{0.4781404745760385, 0.8304391288619981}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.36990399808618973`], 
         DiskBox[{1.3991853397812652`, 0.}, 0.14742528472252805`, {
          0, 3.141592653589793}], 
         {Opacity[0.03220951722406651], 
          DiskBox[{1.3991853397812652`, 0.}, 0.14742528472252805`, {
           3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.4170674250719294], 
         DiskBox[{1.9202254717071263`, 1.7920472987842249`}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[0.29891950042274307`], 
          DiskBox[{1.9202254717071263`, 1.7920472987842249`}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.07162983824482305], 
         DiskBox[{0., 1.5294776837065591`}, 0.14742528472252805`, {
          0, 3.141592653589793}], 
         {Opacity[0.0046557028994501535`], 
          DiskBox[{0., 1.5294776837065591`}, 0.14742528472252805`, {
           3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.4893507402506109], 
         DiskBox[{1.6883121489539397`, 0.7643945280875081}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[0.4448029431940553], 
          DiskBox[{1.6883121489539397`, 0.7643945280875081}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.2893431770031411], 
         DiskBox[{1.1532983406755593`, 1.2143678573508532`}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[0.4970330946629057], 
          DiskBox[{1.1532983406755593`, 1.2143678573508532`}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}}, 
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], LineBox[CompressedData["
1:eJwBdQGK/iFib1JlAwAAAAsAAAACAAAAAgAAACquB4jeFgJAvSWX85iK9j/K
oDmT+wAAQKuNA8nkU/o/kAjWApbWAUBLN5w39g7yPwbYSnKdzPw/x8/D+Ya8
7D+faOpLEXgBQGU58HpH6vI/IquKPDz69D8aPtrXLlryPyumLe58WNo/sYW8
JjFj7z+7F3xMdgWxP2zwVY6fEPY/9kJ1mYpW5D/7c1skufDnPwevK+2Efvg/
rwh6OHJg5j8MNMCP9ojjP20GE9GKau4/tMluTuRV8D/bJj0fQoLxP0gEYRt2
APc/dZwzGGbuwz+flOio7WX6P+4qX/1ReuM/IkJwE94T9j/ibYxajizEP/3p
QQ0bw/I/UJ04Mnvo8D8cba8v6V7+P3/6meVwJ/o/m5ICwqhd+z+AV91LfX/t
P35R8M1I+vw/hpNEzIyD+j9yQSqA3jL0P8A0GPu5lvU/Qgar1EmLwz++Zdiu
4e/2P+pXZrt/AvA/inkp5uj29D/LArPg
         "]]}}], {}},
    FrameTicks->None], ",", 
   GraphicsBox[{GraphicsGroupBox[{
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.0008139586421020535], 
         DiskBox[{2.341421002111228, 1.2622718318610946`}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[0.07864887313230348], 
          DiskBox[{2.341421002111228, 1.2622718318610946`}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.00736591674056535], 
         DiskBox[{0.4781404745760385, 0.8304391288619981}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[0.21910319895732097`], 
          DiskBox[{0.4781404745760385, 0.8304391288619981}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.03220951722406651], 
         DiskBox[{1.3991853397812652`, 0.}, 0.14742528472252805`, {
          0, 3.141592653589793}], 
         {Opacity[0.18443231528287604`], 
          DiskBox[{1.3991853397812652`, 0.}, 0.14742528472252805`, {
           3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.29891950042274307`], 
         DiskBox[{1.9202254717071263`, 1.7920472987842249`}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[0.22052777903158885`], 
          DiskBox[{1.9202254717071263`, 1.7920472987842249`}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.0046557028994501535`], 
         DiskBox[{0., 1.5294776837065591`}, 0.14742528472252805`, {
          0, 3.141592653589793}], 
         {Opacity[0.021351930796048423`], 
          DiskBox[{0., 1.5294776837065591`}, 0.14742528472252805`, {
           3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.4448029431940553], 
         DiskBox[{1.6883121489539397`, 0.7643945280875081}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[0.00024181003768195732`], 
          DiskBox[{1.6883121489539397`, 0.7643945280875081}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.4970330946629057], 
         DiskBox[{1.1532983406755593`, 1.2143678573508532`}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[0.011717941136212712`], 
          DiskBox[{1.1532983406755593`, 1.2143678573508532`}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}}, 
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], LineBox[CompressedData["
1:eJwBdQGK/iFib1JlAwAAAAsAAAACAAAAAgAAACquB4jeFgJAvSWX85iK9j/K
oDmT+wAAQKuNA8nkU/o/kAjWApbWAUBLN5w39g7yPwbYSnKdzPw/x8/D+Ya8
7D+faOpLEXgBQGU58HpH6vI/IquKPDz69D8aPtrXLlryPyumLe58WNo/sYW8
JjFj7z+7F3xMdgWxP2zwVY6fEPY/9kJ1mYpW5D/7c1skufDnPwevK+2Efvg/
rwh6OHJg5j8MNMCP9ojjP20GE9GKau4/tMluTuRV8D/bJj0fQoLxP0gEYRt2
APc/dZwzGGbuwz+flOio7WX6P+4qX/1ReuM/IkJwE94T9j/ibYxajizEP/3p
QQ0bw/I/UJ04Mnvo8D8cba8v6V7+P3/6meVwJ/o/m5ICwqhd+z+AV91LfX/t
P35R8M1I+vw/hpNEzIyD+j9yQSqA3jL0P8A0GPu5lvU/Qgar1EmLwz++Zdiu
4e/2P+pXZrt/AvA/inkp5uj29D/LArPg
         "]]}}], {}},
    FrameTicks->None], ",", 
   GraphicsBox[{GraphicsGroupBox[{
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.07864887313230348], 
         DiskBox[{2.341421002111228, 1.2622718318610946`}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[0.07288377016838911], 
          DiskBox[{2.341421002111228, 1.2622718318610946`}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.21910319895732097`], 
         DiskBox[{0.4781404745760385, 0.8304391288619981}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[0.048781201347934844`], 
          DiskBox[{0.4781404745760385, 0.8304391288619981}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.18443231528287604`], 
         DiskBox[{1.3991853397812652`, 0.}, 0.14742528472252805`, {
          0, 3.141592653589793}], 
         {Opacity[0.00039675486392730543`], 
          DiskBox[{1.3991853397812652`, 0.}, 0.14742528472252805`, {
           3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.22052777903158885`], 
         DiskBox[{1.9202254717071263`, 1.7920472987842249`}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[1.7599879152449656`*^-11], 
          DiskBox[{1.9202254717071263`, 1.7920472987842249`}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.021351930796048423`], 
         DiskBox[{0., 1.5294776837065591`}, 0.14742528472252805`, {
          0, 3.141592653589793}], 
         {Opacity[0.019283987395789792`], 
          DiskBox[{0., 1.5294776837065591`}, 0.14742528472252805`, {
           3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.00024181003768195732`], 
         DiskBox[{1.6883121489539397`, 0.7643945280875081}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[0.4993292951854014], 
          DiskBox[{1.6883121489539397`, 0.7643945280875081}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.011717941136212712`], 
         DiskBox[{1.1532983406755593`, 1.2143678573508532`}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[0.49999963348015014`], 
          DiskBox[{1.1532983406755593`, 1.2143678573508532`}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}}, 
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], LineBox[CompressedData["
1:eJwBdQGK/iFib1JlAwAAAAsAAAACAAAAAgAAACquB4jeFgJAvSWX85iK9j/K
oDmT+wAAQKuNA8nkU/o/kAjWApbWAUBLN5w39g7yPwbYSnKdzPw/x8/D+Ya8
7D+faOpLEXgBQGU58HpH6vI/IquKPDz69D8aPtrXLlryPyumLe58WNo/sYW8
JjFj7z+7F3xMdgWxP2zwVY6fEPY/9kJ1mYpW5D/7c1skufDnPwevK+2Efvg/
rwh6OHJg5j8MNMCP9ojjP20GE9GKau4/tMluTuRV8D/bJj0fQoLxP0gEYRt2
APc/dZwzGGbuwz+flOio7WX6P+4qX/1ReuM/IkJwE94T9j/ibYxajizEP/3p
QQ0bw/I/UJ04Mnvo8D8cba8v6V7+P3/6meVwJ/o/m5ICwqhd+z+AV91LfX/t
P35R8M1I+vw/hpNEzIyD+j9yQSqA3jL0P8A0GPu5lvU/Qgar1EmLwz++Zdiu
4e/2P+pXZrt/AvA/inkp5uj29D/LArPg
         "]]}}], {}},
    FrameTicks->None], ",", 
   GraphicsBox[{GraphicsGroupBox[{
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.07288377016838911], 
         DiskBox[{2.341421002111228, 1.2622718318610946`}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[0.0007910570985513836], 
          DiskBox[{2.341421002111228, 1.2622718318610946`}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.048781201347934844`], 
         DiskBox[{0.4781404745760385, 0.8304391288619981}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[0.00516148226173372], 
          DiskBox[{0.4781404745760385, 0.8304391288619981}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.00039675486392730543`], 
         DiskBox[{1.3991853397812652`, 0.}, 0.14742528472252805`, {
          0, 3.141592653589793}], 
         {Opacity[1.7365908662286542`*^-11], 
          DiskBox[{1.3991853397812652`, 0.}, 0.14742528472252805`, {
           3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[1.7599879152449656`*^-11], 
         DiskBox[{1.9202254717071263`, 1.7920472987842249`}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[4.55896796261704*^-24], 
          DiskBox[{1.9202254717071263`, 1.7920472987842249`}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.019283987395789792`], 
         DiskBox[{0., 1.5294776837065591`}, 0.14742528472252805`, {
          0, 3.141592653589793}], 
         {Opacity[0.0019295656972580441`], 
          DiskBox[{0., 1.5294776837065591`}, 0.14742528472252805`, {
           3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.4993292951854014], 
         DiskBox[{1.6883121489539397`, 0.7643945280875081}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[0.4991405771191728], 
          DiskBox[{1.6883121489539397`, 0.7643945280875081}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.49999963348015014`], 
         DiskBox[{1.1532983406755593`, 1.2143678573508532`}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[0.5], 
          DiskBox[{1.1532983406755593`, 1.2143678573508532`}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}}, 
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], LineBox[CompressedData["
1:eJwBdQGK/iFib1JlAwAAAAsAAAACAAAAAgAAACquB4jeFgJAvSWX85iK9j/K
oDmT+wAAQKuNA8nkU/o/kAjWApbWAUBLN5w39g7yPwbYSnKdzPw/x8/D+Ya8
7D+faOpLEXgBQGU58HpH6vI/IquKPDz69D8aPtrXLlryPyumLe58WNo/sYW8
JjFj7z+7F3xMdgWxP2zwVY6fEPY/9kJ1mYpW5D/7c1skufDnPwevK+2Efvg/
rwh6OHJg5j8MNMCP9ojjP20GE9GKau4/tMluTuRV8D/bJj0fQoLxP0gEYRt2
APc/dZwzGGbuwz+flOio7WX6P+4qX/1ReuM/IkJwE94T9j/ibYxajizEP/3p
QQ0bw/I/UJ04Mnvo8D8cba8v6V7+P3/6meVwJ/o/m5ICwqhd+z+AV91LfX/t
P35R8M1I+vw/hpNEzIyD+j9yQSqA3jL0P8A0GPu5lvU/Qgar1EmLwz++Zdiu
4e/2P+pXZrt/AvA/inkp5uj29D/LArPg
         "]]}}], {}},
    FrameTicks->None], ",", 
   GraphicsBox[{GraphicsGroupBox[{
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.0007910570985513836], 
         DiskBox[{2.341421002111228, 1.2622718318610946`}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[0.07857667266255401], 
          DiskBox[{2.341421002111228, 1.2622718318610946`}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.00516148226173372], 
         DiskBox[{0.4781404745760385, 0.8304391288619981}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[0.2188726756318221], 
          DiskBox[{0.4781404745760385, 0.8304391288619981}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[1.7365908662286542`*^-11], 
         DiskBox[{1.3991853397812652`, 0.}, 0.14742528472252805`, {
          0, 3.141592653589793}], 
         {Opacity[4.910368234951121*^-17], 
          DiskBox[{1.3991853397812652`, 0.}, 0.14742528472252805`, {
           3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[4.55896796261704*^-24], 
         DiskBox[{1.9202254717071263`, 1.7920472987842249`}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[1.4138990945950923`*^-110], 
          DiskBox[{1.9202254717071263`, 1.7920472987842249`}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.0019295656972580441`], 
         DiskBox[{0., 1.5294776837065591`}, 0.14742528472252805`, {
          0, 3.141592653589793}], 
         {Opacity[0.01070291207123125], 
          DiskBox[{0., 1.5294776837065591`}, 0.14742528472252805`, {
           3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.4991405771191728], 
         DiskBox[{1.6883121489539397`, 0.7643945280875081}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[0.4993309033442941], 
          DiskBox[{1.6883121489539397`, 0.7643945280875081}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.5], 
         DiskBox[{1.1532983406755593`, 1.2143678573508532`}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[0.4999999999999968], 
          DiskBox[{1.1532983406755593`, 1.2143678573508532`}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}}, 
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], LineBox[CompressedData["
1:eJwBdQGK/iFib1JlAwAAAAsAAAACAAAAAgAAACquB4jeFgJAvSWX85iK9j/K
oDmT+wAAQKuNA8nkU/o/kAjWApbWAUBLN5w39g7yPwbYSnKdzPw/x8/D+Ya8
7D+faOpLEXgBQGU58HpH6vI/IquKPDz69D8aPtrXLlryPyumLe58WNo/sYW8
JjFj7z+7F3xMdgWxP2zwVY6fEPY/9kJ1mYpW5D/7c1skufDnPwevK+2Efvg/
rwh6OHJg5j8MNMCP9ojjP20GE9GKau4/tMluTuRV8D/bJj0fQoLxP0gEYRt2
APc/dZwzGGbuwz+flOio7WX6P+4qX/1ReuM/IkJwE94T9j/ibYxajizEP/3p
QQ0bw/I/UJ04Mnvo8D8cba8v6V7+P3/6meVwJ/o/m5ICwqhd+z+AV91LfX/t
P35R8M1I+vw/hpNEzIyD+j9yQSqA3jL0P8A0GPu5lvU/Qgar1EmLwz++Zdiu
4e/2P+pXZrt/AvA/inkp5uj29D/LArPg
         "]]}}], {}},
    FrameTicks->None], ",", 
   GraphicsBox[{GraphicsGroupBox[{
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.07857667266255401], 
         DiskBox[{2.341421002111228, 1.2622718318610946`}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[0.07261795861229246], 
          DiskBox[{2.341421002111228, 1.2622718318610946`}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.2188726756318221], 
         DiskBox[{0.4781404745760385, 0.8304391288619981}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[0.04249206629281657], 
          DiskBox[{0.4781404745760385, 0.8304391288619981}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[4.910368234951121*^-17], 
         DiskBox[{1.3991853397812652`, 0.}, 0.14742528472252805`, {
          0, 3.141592653589793}], 
         {Opacity[7.038882695111297*^-60], 
          DiskBox[{1.3991853397812652`, 0.}, 0.14742528472252805`, {
           3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[1.4138990945950923`*^-110], 
         DiskBox[{1.9202254717071263`, 1.7920472987842249`}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[
          8.83175501619656446664704616463094584465420285`15.954589770191005*^-\
399], DiskBox[{1.9202254717071263`, 1.7920472987842249`}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.01070291207123125], 
         DiskBox[{0., 1.5294776837065591`}, 0.14742528472252805`, {
          0, 3.141592653589793}], 
         {Opacity[0.005018791916266954], 
          DiskBox[{0., 1.5294776837065591`}, 0.14742528472252805`, {
           3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.4993309033442941], 
         DiskBox[{1.6883121489539397`, 0.7643945280875081}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[0.4993296219274685], 
          DiskBox[{1.6883121489539397`, 0.7643945280875081}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.4999999999999968], 
         DiskBox[{1.1532983406755593`, 1.2143678573508532`}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[0.5], 
          DiskBox[{1.1532983406755593`, 1.2143678573508532`}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}}, 
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], LineBox[CompressedData["
1:eJwBdQGK/iFib1JlAwAAAAsAAAACAAAAAgAAACquB4jeFgJAvSWX85iK9j/K
oDmT+wAAQKuNA8nkU/o/kAjWApbWAUBLN5w39g7yPwbYSnKdzPw/x8/D+Ya8
7D+faOpLEXgBQGU58HpH6vI/IquKPDz69D8aPtrXLlryPyumLe58WNo/sYW8
JjFj7z+7F3xMdgWxP2zwVY6fEPY/9kJ1mYpW5D/7c1skufDnPwevK+2Efvg/
rwh6OHJg5j8MNMCP9ojjP20GE9GKau4/tMluTuRV8D/bJj0fQoLxP0gEYRt2
APc/dZwzGGbuwz+flOio7WX6P+4qX/1ReuM/IkJwE94T9j/ibYxajizEP/3p
QQ0bw/I/UJ04Mnvo8D8cba8v6V7+P3/6meVwJ/o/m5ICwqhd+z+AV91LfX/t
P35R8M1I+vw/hpNEzIyD+j9yQSqA3jL0P8A0GPu5lvU/Qgar1EmLwz++Zdiu
4e/2P+pXZrt/AvA/inkp5uj29D/LArPg
         "]]}}], {}},
    FrameTicks->None], ",", 
   GraphicsBox[{GraphicsGroupBox[{
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.07261795861229246], 
         DiskBox[{2.341421002111228, 1.2622718318610946`}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[0.0007711889623961556], 
          DiskBox[{2.341421002111228, 1.2622718318610946`}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.04249206629281657], 
         DiskBox[{0.4781404745760385, 0.8304391288619981}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[0.004265384852759786], 
          DiskBox[{0.4781404745760385, 0.8304391288619981}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[7.038882695111297*^-60], 
         DiskBox[{1.3991853397812652`, 0.}, 0.14742528472252805`, {
          0, 3.141592653589793}], 
         {Opacity[1.871092858556119*^-99], 
          DiskBox[{1.3991853397812652`, 0.}, 0.14742528472252805`, {
           3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[8.83175501619656446664704616463094584465420285`15.\
954589770191005*^-399], 
         DiskBox[{1.9202254717071263`, 1.7920472987842249`}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[
          1.7345583602875991725738550009556883078137635`15.954589770191005*^-\
1248], DiskBox[{1.9202254717071263`, 1.7920472987842249`}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.005018791916266954], 
         DiskBox[{0., 1.5294776837065591`}, 0.14742528472252805`, {
          0, 3.141592653589793}], 
         {Opacity[0.0004831582405286675], 
          DiskBox[{0., 1.5294776837065591`}, 0.14742528472252805`, {
           3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.4993296219274685], 
         DiskBox[{1.6883121489539397`, 0.7643945280875081}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[0.4997848669443876], 
          DiskBox[{1.6883121489539397`, 0.7643945280875081}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.5], 
         DiskBox[{1.1532983406755593`, 1.2143678573508532`}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[0.5], 
          DiskBox[{1.1532983406755593`, 1.2143678573508532`}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}}, 
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], LineBox[CompressedData["
1:eJwBdQGK/iFib1JlAwAAAAsAAAACAAAAAgAAACquB4jeFgJAvSWX85iK9j/K
oDmT+wAAQKuNA8nkU/o/kAjWApbWAUBLN5w39g7yPwbYSnKdzPw/x8/D+Ya8
7D+faOpLEXgBQGU58HpH6vI/IquKPDz69D8aPtrXLlryPyumLe58WNo/sYW8
JjFj7z+7F3xMdgWxP2zwVY6fEPY/9kJ1mYpW5D/7c1skufDnPwevK+2Efvg/
rwh6OHJg5j8MNMCP9ojjP20GE9GKau4/tMluTuRV8D/bJj0fQoLxP0gEYRt2
APc/dZwzGGbuwz+flOio7WX6P+4qX/1ReuM/IkJwE94T9j/ibYxajizEP/3p
QQ0bw/I/UJ04Mnvo8D8cba8v6V7+P3/6meVwJ/o/m5ICwqhd+z+AV91LfX/t
P35R8M1I+vw/hpNEzIyD+j9yQSqA3jL0P8A0GPu5lvU/Qgar1EmLwz++Zdiu
4e/2P+pXZrt/AvA/inkp5uj29D/LArPg
         "]]}}], {}},
    FrameTicks->None], ",", 
   GraphicsBox[{GraphicsGroupBox[{
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.0007711889623961556], 
         DiskBox[{2.341421002111228, 1.2622718318610946`}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[0.07853381995480561], 
          DiskBox[{2.341421002111228, 1.2622718318610946`}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.004265384852759786], 
         DiskBox[{0.4781404745760385, 0.8304391288619981}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[0.2187040512310523], 
          DiskBox[{0.4781404745760385, 0.8304391288619981}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[1.871092858556119*^-99], 
         DiskBox[{1.3991853397812652`, 0.}, 0.14742528472252805`, {
          0, 3.141592653589793}], 
         {Opacity[7.372717183151343*^-219], 
          DiskBox[{1.3991853397812652`, 0.}, 0.14742528472252805`, {
           3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[1.7345583602875991725738550009556883078137635`15.\
954589770191005*^-1248], 
         DiskBox[{1.9202254717071263`, 1.7920472987842249`}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[
          1.8386192461196115270448509584741679810492497`15.954589770191005*^-\
4138], DiskBox[{1.9202254717071263`, 1.7920472987842249`}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.0004831582405286675], 
         DiskBox[{0., 1.5294776837065591`}, 0.14742528472252805`, {
          0, 3.141592653589793}], 
         {Opacity[0.002738042764195353], 
          DiskBox[{0., 1.5294776837065591`}, 0.14742528472252805`, {
           3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.4997848669443876], 
         DiskBox[{1.6883121489539397`, 0.7643945280875081}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[0.4998325577835329], 
          DiskBox[{1.6883121489539397`, 0.7643945280875081}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}, 
        {RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.5], 
         DiskBox[{1.1532983406755593`, 1.2143678573508532`}, 
          0.14742528472252805`, {0, 3.141592653589793}], 
         {Opacity[0.5], 
          DiskBox[{1.1532983406755593`, 1.2143678573508532`}, 
           0.14742528472252805`, {3.141592653589793, 6.283185307179586}]}}}, 
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], LineBox[CompressedData["
1:eJwBdQGK/iFib1JlAwAAAAsAAAACAAAAAgAAACquB4jeFgJAvSWX85iK9j/K
oDmT+wAAQKuNA8nkU/o/kAjWApbWAUBLN5w39g7yPwbYSnKdzPw/x8/D+Ya8
7D+faOpLEXgBQGU58HpH6vI/IquKPDz69D8aPtrXLlryPyumLe58WNo/sYW8
JjFj7z+7F3xMdgWxP2zwVY6fEPY/9kJ1mYpW5D/7c1skufDnPwevK+2Efvg/
rwh6OHJg5j8MNMCP9ojjP20GE9GKau4/tMluTuRV8D/bJj0fQoLxP0gEYRt2
APc/dZwzGGbuwz+flOio7WX6P+4qX/1ReuM/IkJwE94T9j/ibYxajizEP/3p
QQ0bw/I/UJ04Mnvo8D8cba8v6V7+P3/6meVwJ/o/m5ICwqhd+z+AV91LfX/t
P35R8M1I+vw/hpNEzIyD+j9yQSqA3jL0P8A0GPu5lvU/Qgar1EmLwz++Zdiu
4e/2P+pXZrt/AvA/inkp5uj29D/LArPg
         "]]}}], {}},
    FrameTicks->None]}], "}"}]], "Output",
 CellChangeTimes->{3.506053036928624*^9, 3.5060536083379107`*^9, 
  3.506053643915389*^9, 3.506053692927454*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"gname", "=", 
   RowBox[{"{", 
    RowBox[{"7", ",", "310"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"g", "=", 
   RowBox[{"GraphData", "[", "gname", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"\[Lambda]", "=", "50."}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"initHardcore", "@", "g"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"decompBethe", "=", 
   RowBox[{"findTreeDecomposition", "@", "g"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"initMessages", "@", "decompBethe"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"parallelUpdate", "[", "10", "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"visualizeCavities2", "[", "]"}]}], "Input",
 CellChangeTimes->{{3.506055050124652*^9, 3.5060551048945923`*^9}, {
  3.5060551476749563`*^9, 3.50605520904173*^9}, {3.5060552503671083`*^9, 
  3.506055329259755*^9}, {3.506055380336158*^9, 3.506055418337219*^9}, {
  3.506055463590803*^9, 3.506055574310313*^9}, {3.5060556272578917`*^9, 
  3.50605563509555*^9}}],

Cell[BoxData["50.`"], "Output",
 CellChangeTimes->{{3.5060550789350843`*^9, 3.506055105218854*^9}, {
   3.50605514843566*^9, 3.506055209310895*^9}, {3.506055250998356*^9, 
   3.506055329545768*^9}, {3.506055391412252*^9, 3.506055418615485*^9}, {
   3.506055466889385*^9, 3.506055544020199*^9}, 3.5060555746531057`*^9, {
   3.506055629677286*^9, 3.506055635410556*^9}}],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5, 6, 7}, {
      Null, {{1, 5}, {1, 6}, {1, 7}, {2, 5}, {2, 6}, {3, 5}, {3, 7}, {4, 6}, {
       4, 7}, {5, 7}}}, {
      VertexShapeFunction -> {$CellContext`vf}, 
       VertexSize -> {{"Scaled", 0.1}}, 
       VertexCoordinates -> {{1.5512988565146848`, 0.960769627630215}, {
        1.9381339952911445`, 1.92162111568225}, {0., 0.9613158719848925}, {
        1.9377277894001554`, 0.}, {0.8821429958201967, 1.4378659257091104`}, {
        2.4098985401588067`, 0.9608654190142425}, {0.8817310200857476, 
        0.4836395586713559}}}]], Typeset`boxes = GraphicsGroupBox[{{
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[{
         RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.9785623791375763], 
         DiskBox[{1.5512988565146848`, 0.960769627630215}, 0.1541122860619445, 
          NCache[{0, 2 Pi}, {0, 6.283185307179586}]]}, "DynamicName", BoxID -> 
        "VertexID$1"], 
       TagBox[{
         RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.9789312276118198], 
         DiskBox[{1.9381339952911445`, 1.92162111568225}, 0.1541122860619445, 
          
          NCache[{0, 2 Pi}, {0, 6.283185307179586}]]}, "DynamicName", BoxID -> 
        "VertexID$2"], 
       TagBox[{
         RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.9789312276118198], 
         DiskBox[{0., 0.9613158719848925}, 0.1541122860619445, 
          NCache[{0, 2 Pi}, {0, 6.283185307179586}]]}, "DynamicName", BoxID -> 
        "VertexID$3"], 
       TagBox[{
         RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.9789312276118198], 
         DiskBox[{1.9377277894001554`, 0.}, 0.1541122860619445, 
          NCache[{0, 2 Pi}, {0, 6.283185307179586}]]}, "DynamicName", BoxID -> 
        "VertexID$4"], 
       TagBox[{
         RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.0007450739179719354], 
         DiskBox[{0.8821429958201967, 1.4378659257091104`}, 
          0.1541122860619445, 
          NCache[{0, 2 Pi}, {0, 6.283185307179586}]]}, "DynamicName", BoxID -> 
        "VertexID$5"], 
       TagBox[{
         RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.0011139223922154678`], 
         DiskBox[{2.4098985401588067`, 0.9608654190142425}, 
          0.1541122860619445, 
          NCache[{0, 2 Pi}, {0, 6.283185307179586}]]}, "DynamicName", BoxID -> 
        "VertexID$6"], 
       TagBox[{
         RGBColor[
         0.09803921568627451, 0.4549019607843137, 0.8235294117647058], 
         Opacity[0.0007450739179719354], 
         DiskBox[{0.8817310200857476, 0.4836395586713559}, 0.1541122860619445, 
          NCache[{0, 2 Pi}, {0, 6.283185307179586}]]}, "DynamicName", BoxID -> 
        "VertexID$7"]}, {
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], 
       LineBox[{{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}, {
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$6", Automatic, Center]}, {
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$7", Automatic, Center]}, {
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}, {
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$6", Automatic, Center]}, {
          DynamicLocation["VertexID$3", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}, {
          DynamicLocation["VertexID$3", Automatic, Center], 
          DynamicLocation["VertexID$7", Automatic, Center]}, {
          DynamicLocation["VertexID$4", Automatic, Center], 
          DynamicLocation["VertexID$6", Automatic, Center]}, {
          DynamicLocation["VertexID$4", Automatic, Center], 
          DynamicLocation["VertexID$7", Automatic, Center]}, {
          DynamicLocation["VertexID$5", Automatic, Center], 
          DynamicLocation["VertexID$7", Automatic, Center]}}]}}]}, 
    DynamicBox[GraphComputation`NetworkGraphicsBox[
     1, Typeset`graph, Typeset`boxes], {
     CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
      SynchronousUpdating -> False, TrackedSymbols :> {}},
     ImageSizeCache->{{6.18, 352.82}, {-145.10719999999998`, 
      139.62245796612575`}}]]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None]], "Output",
 CellChangeTimes->{{3.5060550789350843`*^9, 3.506055105218854*^9}, {
   3.50605514843566*^9, 3.506055209310895*^9}, {3.506055250998356*^9, 
   3.506055329545768*^9}, {3.506055391412252*^9, 3.506055418615485*^9}, {
   3.506055466889385*^9, 3.506055544020199*^9}, 3.5060555746531057`*^9, {
   3.506055629677286*^9, 3.506055635706728*^9}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Animation", "Section",
 CellChangeTimes->{{3.506056295662245*^9, 3.506056298921*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"doTemp", "[", "x_", "]"}], ":=", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"\[Lambda]", "=", "x"}], ";", "\[IndentingNewLine]", 
      RowBox[{"initHardcore", "@", "g"}], ";", "\[IndentingNewLine]", 
      RowBox[{"decompBethe", "=", 
       RowBox[{"findTreeDecomposition", "@", "g"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"initMessages", "@", "decompBethe"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"parallelUpdate", "[", "10", "]"}], ";", "\[IndentingNewLine]", 
      
      RowBox[{"visualizeCavities2", "[", "]"}]}], "\[IndentingNewLine]", 
     ")"}]}], ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"pics", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"doTemp", "[", "xx", "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"xx", ",", "0", ",", "3", ",", ".02"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{"\"\<~/Desktop/cavities.gif\>\"", ",", "pics"}], "]"}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.506056311706147*^9, 3.506056336420603*^9}}]
}, Open  ]]
},
WindowSize->{740, 752},
WindowMargins->{{71, Automatic}, {Automatic, 20}},
InputAliases->{"intt" -> RowBox[{"\[Integral]", 
     RowBox[{"\[SelectionPlaceholder]", 
       RowBox[{"\[DifferentialD]", "\[Placeholder]"}]}]}], "dintt" -> 
  RowBox[{
     SubsuperscriptBox[
     "\[Integral]", "\[SelectionPlaceholder]", "\[Placeholder]"], 
     RowBox[{"\[Placeholder]", 
       RowBox[{"\[DifferentialD]", "\[Placeholder]"}]}]}], "sumt" -> RowBox[{
     UnderoverscriptBox["\[Sum]", 
      RowBox[{"\[SelectionPlaceholder]", "=", "\[Placeholder]"}], 
      "\[Placeholder]"], "\[Placeholder]"}], "prodt" -> RowBox[{
     UnderoverscriptBox["\[Product]", 
      RowBox[{"\[SelectionPlaceholder]", "=", "\[Placeholder]"}], 
      "\[Placeholder]"], "\[Placeholder]"}], "dt" -> RowBox[{
     SubscriptBox["\[PartialD]", "\[Placeholder]"], " ", 
     "\[SelectionPlaceholder]"}], "ket" -> 
  TemplateBox[{"\[Placeholder]"}, "Ket"], "bra" -> 
  TemplateBox[{"\[Placeholder]"}, "Bra"], "braket" -> 
  TemplateBox[{"\[Placeholder]", "\[Placeholder]"}, "BraKet"], "in" -> 
  "\[Element]"},
FrontEndVersion->"8.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (November 6, \
2010)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[579, 22, 140, 2, 67, "Section"],
Cell[722, 26, 449, 13, 73, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1208, 44, 90, 1, 67, "Section"],
Cell[1301, 47, 7653, 193, 598, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[8991, 245, 139, 2, 67, "Section"],
Cell[9133, 249, 344, 7, 41, "Text"],
Cell[9480, 258, 9923, 241, 988, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[19440, 504, 146, 2, 67, "Section"],
Cell[19589, 508, 8539, 219, 808, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[28165, 732, 106, 1, 67, "Section"],
Cell[28274, 735, 16756, 422, 1348, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[45067, 1162, 92, 1, 37, "Section"],
Cell[45162, 1165, 21204, 544, 2083, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[66403, 1714, 92, 1, 37, "Section"],
Cell[CellGroupData[{
Cell[66520, 1719, 727, 18, 103, "Input"],
Cell[67250, 1739, 315, 10, 27, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[67602, 1754, 1524, 48, 88, "Input"],
Cell[69129, 1804, 1063, 33, 27, "Output"]
}, Open  ]],
Cell[70207, 1840, 808, 19, 88, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[71052, 1864, 89, 1, 37, "Section"],
Cell[CellGroupData[{
Cell[71166, 1869, 1524, 41, 193, "Input"],
Cell[72693, 1912, 213, 3, 30, "Output"],
Cell[72909, 1917, 195, 3, 27, "Output"],
Cell[73107, 1922, 208, 3, 27, "Output"],
Cell[73318, 1927, 192, 3, 27, "Output"]
}, Open  ]],
Cell[73525, 1933, 764, 23, 103, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[74326, 1961, 92, 1, 37, "Section"],
Cell[74421, 1964, 907, 23, 88, "Input"],
Cell[CellGroupData[{
Cell[75353, 1991, 784, 20, 118, "Input"],
Cell[76140, 2013, 13897, 280, 387, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[90074, 2298, 953, 27, 118, "Input"],
Cell[91030, 2327, 37353, 709, 665, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[128420, 3041, 1057, 25, 133, "Input"],
Cell[129480, 3068, 368, 5, 27, "Output"],
Cell[129851, 3075, 5285, 109, 313, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[135185, 3190, 91, 1, 67, "Section"],
Cell[135279, 3193, 1166, 30, 178, "Input"]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
